(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[  23026657,     654475]
NotebookOptionsPosition[  23001435,     653910]
NotebookOutlinePosition[  23001933,     653928]
CellTagsIndexPosition[  23001890,     653925]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Tables of Branching Rules", "Title",ExpressionUUID->"0b4c7478-88ab-43fa-8576-f7e31bd8b9e7"],

Cell[TextData[{
 "following R. Slansky, ",
 StyleBox["\"Group Theory for Unified Model Building\"",
  FontSlant->"Italic"],
 ", Phys.Rept. ",
 StyleBox["79",
  FontWeight->"Bold"],
 " (1981) 1 - 128"
}], "Subsubtitle",ExpressionUUID->"d7b83ecd-1c48-45de-a6a5-8ed6a04c7a0f"],

Cell[CellGroupData[{

Cell["Initialization", "Section",ExpressionUUID->"dcc38c6f-cfa2-42c9-aea8-6d499dbb4386"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"<<", "LieART`Tables`"}]], "Input",
 InitializationCell->
  True,ExpressionUUID->"381f412f-111f-410a-9c54-2f74acb1dfa4"],

Cell[BoxData[
 FormBox["\<\"LieART 1.1.4\"\>", TraditionalForm]], "Print",
 CellChangeTimes->{3.616237062650613*^9, 3.616238784339279*^9, 
  3.616299468971784*^9},ExpressionUUID->"f6bd3fbb-c159-41f4-af76-\
d0506601f42b"],

Cell[BoxData[
 FormBox["\<\"last revised 5 August 2014\"\>", TraditionalForm]], "Print",
 CellChangeTimes->{3.616237062650613*^9, 3.616238784339279*^9, 
  3.6162994689724197`*^9},ExpressionUUID->"384d90db-6c8b-4c42-b504-\
2f871ea0a691"]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"$WriteToLaTeXFile", "=", "False"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
  "$BranchingRulesLaTeXFile", "=", "\"\<BranchingRulesTables.tex\>\""}], 
  ";"}]}], "Input",
 InitializationCell->
  True,ExpressionUUID->"90bcb6f0-ae5d-4ed1-9002-bfa594defbe0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"FileExistsQ", "[", "$BranchingRulesLaTeXFile", "]"}], ",", 
    RowBox[{"DeleteFile", "[", "$BranchingRulesLaTeXFile", "]"}]}], "]"}], 
  ";"}]], "Input",ExpressionUUID->"b8b56ffc-5773-4fec-bf57-0f3b55946d09"]
}, Closed]],

Cell[CellGroupData[{

Cell["SU(N)", "Section",ExpressionUUID->"b3b0012b-3662-4eca-8162-00d136cab5da"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU2", ",", 
   RowBox[{"{", "U1", "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "29"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "28"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.782326970380808*^9, 3.7823269806284256`*^9}, {
  3.783673614996014*^9, 3.783673619614962*^9}, {3.7836737276692877`*^9, 
  3.783673731873048*^9}},ExpressionUUID->"2b166e86-bbec-49a3-b1fc-\
6e280ac57893"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"0.1\"\>",
      0.1191275,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[0.1191275, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.615255836200057*^9, {3.782326973880438*^9, 3.7823269816118*^9}, 
   3.78367362367078*^9, 
   3.783673742267005*^9},ExpressionUUID->"2ad7ff5b-4bb5-4cc3-87ae-\
1e4fb253dd6a"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "36", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(2)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"U(1)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"13", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(19)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-19)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(20)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-20)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(21)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(19)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-19)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-21)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(22)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(20)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-20)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-22)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(23)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(21)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(19)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-19)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-21)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-23)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(24)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(22)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(20)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-20)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-22)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-24)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(25)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(23)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(21)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(19)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-19)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-21)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-23)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-25)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(26)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(24)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(22)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(20)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-20)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-22)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-24)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-26)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(27)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(25)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(23)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(21)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(19)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-19)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-21)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-23)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-25)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-27)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(28)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(26)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(24)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(22)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(20)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-20)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-22)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-24)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-26)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-28)\"\>",
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.615255836265291*^9, {3.782326973969205*^9, 3.7823269817085433`*^9}, 
   3.7836736238353405`*^9, 
   3.7836737423916717`*^9},ExpressionUUID->"ed03fde6-02a1-4ca6-8a37-\
6fe46f57cc4c"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU3", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "U1"}], "]"}], ",", "SU2"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "132"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "15"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7745636572464857`*^9, 3.774563658302638*^9}, {
   3.7823272733765783`*^9, 3.78232728432032*^9}, {3.7836737761500087`*^9, 
   3.783673793438961*^9}, 3.783673891559434*^9, {3.783673977283496*^9, 
   3.7836739928474035`*^9}},ExpressionUUID->"894fe9eb-3835-4c8c-a404-\
97c7f89d598d"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"0.6\"\>",
      0.5602689,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[0.5602689, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.615255836586982*^9, 3.7776292380532713`*^9, 3.7823270078416476`*^9, {
   3.7823272753023973`*^9, 3.782327285211944*^9}, {3.7836737784937344`*^9, 
   3.783673794476187*^9}, 3.7836738927422457`*^9, {3.7836739786737795`*^9, 
   3.7836739939085665`*^9}},ExpressionUUID->"042b0e6d-10c3-4a02-b4a6-\
7a4e5e9fb928"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "45", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(3)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(3)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"42", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"60", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-26)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"125", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"132", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(3)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["3",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"42", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"60", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"125", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"132", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.6152558366514263`*^9, 3.777629238213809*^9, 3.782327007987288*^9, {
   3.7823272755148625`*^9, 3.78232728565173*^9}, {3.783673778759059*^9, 
   3.783673794761425*^9}, 3.7836738930214977`*^9, {3.7836739789600463`*^9, 
   3.7836739941928353`*^9}},ExpressionUUID->"eab70b92-56f8-41e8-9fbd-\
0eae2a39b1ed"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU4", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "U1"}], "]"}], ",", "Sp4", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "84"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "7"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774563685482855*^9, 3.7745637161139*^9}, {
  3.782327402618103*^9, 3.782327408983081*^9}, {3.7823274976312084`*^9, 
  3.7823275305338464`*^9}, {3.7836741728998613`*^9, 
  3.783674173353648*^9}},ExpressionUUID->"762fc1bc-6111-45a0-a466-\
637c2ab9d288"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"1.2\"\>",
      1.2189407,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[1.2189407, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.6152558377382307`*^9, 3.7776292297845693`*^9, 3.7788379169885597`*^9, {
   3.782327404252699*^9, 3.782327410103059*^9}, {3.782327499971925*^9, 
   3.782327532055747*^9}, 
   3.7836741755158677`*^9},ExpressionUUID->"2afbc7c4-5a16-483c-9493-\
0a739de7859d"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "48", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(4)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(4)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"60", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(4)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"60", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(4)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(4)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["4",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["10",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["20",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SuperscriptBox[
                StyleBox["35",
                 FontWeight->Bold], "\[Prime]"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["56",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"60", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["84",
                FontWeight->Bold],
               TraditionalForm]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(4)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"60", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.615255837884325*^9, 3.777629230106699*^9, 3.7788379173236628`*^9, {
   3.7823274045718756`*^9, 3.782327410473109*^9}, {3.7823275003349886`*^9, 
   3.782327532417801*^9}, 
   3.7836741757841506`*^9},ExpressionUUID->"19b64b38-1fc6-48c8-b4fc-\
9a7c638e37b5"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU5", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "SU2", ",", "U1"}], "]"}], ",", "Sp4"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "210"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "6"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774563751160531*^9, 3.774563752379243*^9}, {
  3.782327708431799*^9, 3.782327776399147*^9}, {3.782327844368252*^9, 
  3.782327861344969*^9}, {3.7836744409818163`*^9, 
  3.7836744594544353`*^9}},ExpressionUUID->"68fe7032-01cf-4af0-a2ef-\
3e3d75947855"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"1.2\"\>",
      1.1636163,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[1.1636163, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.615255838923415*^9, 3.777629218019337*^9, {3.78232783179138*^9, 
   3.7823278624380465`*^9}, {3.7836744460622616`*^9, 
   3.7836744612745657`*^9}},ExpressionUUID->"0e36d88b-941b-4e29-9bc2-\
20866c4658e4"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "52", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(5)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(5)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"40", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"75", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"200", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["84",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(5)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"40", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"75", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"200", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(5)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(4)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["5",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["10",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"40", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"75", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"200", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.6152558390664577`*^9, 3.7776292182217903`*^9, {3.782327831999853*^9, 
   3.782327862658486*^9}, {3.7836744463694124`*^9, 
   3.7836744615318775`*^9}},ExpressionUUID->"b2d48cbd-f2a9-49da-a7b0-\
df088a64261d"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU6", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "SU3", ",", "U1"}], "]"}], ",", "SU3", ",", "SU4", 
     ",", "Sp6", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "SU2"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "384"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774563764037094*^9, 3.7745637942132187`*^9}, {
  3.782328293923322*^9, 3.782328379699599*^9}, {3.7836748385359507`*^9, 
  3.783674838964804*^9}, {3.7836750784100895`*^9, 
  3.7836750791750455`*^9}},ExpressionUUID->"e7faadef-8faf-4e4f-9347-\
a6957d90480d"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"2.4\"\>",
      2.4288019,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[2.4288019, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.6152558417189198`*^9, 3.7776276993852*^9, 3.7788379462843466`*^9, 
   3.7823281180189276`*^9, 3.7823282977670183`*^9, {3.7823283554921784`*^9, 
   3.782328382522053*^9}, 3.783674845156253*^9, 
   3.783675082172033*^9},ExpressionUUID->"960f7e0a-c44a-4b9e-92b6-\
944f6743e601"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "57", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(6)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(3)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox[
                OverscriptBox["6", "_"],
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox[
                OverscriptBox["15", "_"],
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["3", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["15", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["21", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["24", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["42", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["24", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["6", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["15",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["6", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["15",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["15", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["24", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["42", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["63", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["6", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["15",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["66",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["15", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["24", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["42", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["48", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["105", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["3", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["48", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["90", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(4)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["6",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["15",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["70", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["196",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(6)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["6",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["14",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["21",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["56",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["14",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SuperscriptBox[
                StyleBox["126",
                 FontWeight->Bold], "\[Prime]"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["14",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["252",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["126",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["216",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.615255841973948*^9, 3.777627700001538*^9, 3.778837946907645*^9, 
   3.78232811855018*^9, 3.782328298274661*^9, {3.782328355990881*^9, 
   3.7823283830516405`*^9}, 3.78367484574069*^9, 
   3.783675082699623*^9},ExpressionUUID->"5a73be9f-3003-4902-84b0-\
175b508613b3"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU7", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU3", ",", "U1"}], "]"}], ",", "SO7"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "840"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7836751074923763`*^9, 3.7836751211035557`*^9}, {
  3.783675260749139*^9, 3.783675261506115*^9}, {3.783675375789327*^9, 
  3.7836753766629925`*^9}},ExpressionUUID->"af66838a-c06d-474e-a19f-\
662c729d8df1"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"5.7\"\>",
      5.7266226,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[5.7266226, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152558490761423`*^9, 3.7776292060626507`*^9, 
  3.783675129171895*^9, 3.7836752671329513`*^9, 
  3.7836753832902746`*^9},ExpressionUUID->"ed45a218-d538-4db1-806f-\
c0b2074a6079"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "62", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(7)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(7)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"196", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"392", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"588", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"735", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"735", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"756", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"784", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"840", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(7)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"196", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"392", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"588", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"735", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"735", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"756", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"784", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"840", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(7)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"196", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"392", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"588", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"735", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"735", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"756", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"784", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"840", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(7)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(7)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["7",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["21",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["35",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"196", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"392", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["378",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"588", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"735", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"735", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"756", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"784", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"840", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.61525584933768*^9, 3.777629206485481*^9, 
  3.783675129604737*^9, 3.7836752675338817`*^9, 
  3.783675383818861*^9},ExpressionUUID->"ce9bc265-7da2-4eb3-bd3a-\
f866cd82a370"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU8", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU4", ",", "U1"}], "]"}], ",", "SO8", ",", "Sp8", 
     ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU2"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "945"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774563956602369*^9, 3.774563978273407*^9}, {
  3.7823286674406414`*^9, 3.7823286713292465`*^9}, {3.7823287018272924`*^9, 
  3.7823287019888897`*^9}, {3.7823287331844873`*^9, 
  3.782328734085079*^9}},ExpressionUUID->"ed825859-5a01-44b2-a32e-\
0d57e5d2153a"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"17.8\"\>",
      17.7525501,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[17.7525501, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615255873415936*^9, 3.777627979672947*^9, 
  3.7823286934786067`*^9, 3.782328724242366*^9, 
  3.7823287542601514`*^9},ExpressionUUID->"3a0ea1f7-6503-4a21-9926-\
366b37b29423"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "147", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(8)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(35)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(8)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SubscriptBox[
                StyleBox["8",
                 FontWeight->Bold], "v"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["28",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SubscriptBox[
                StyleBox["56",
                 FontWeight->Bold], "v"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "s"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["294",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "cv"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "sv"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubsuperscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "v", "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["294",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(8)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["8",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["36",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["120",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["160",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["160",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["160",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["330",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["308",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["315",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["315",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["160",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["288",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["308",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["315",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SuperscriptBox[
                StyleBox["792",
                 FontWeight->Bold], "\[Prime]\[Prime]"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["315",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.615255873690638*^9, 3.7776279807290936`*^9, 3.78232869403113*^9, {
   3.7823287249395385`*^9, 
   3.782328754870508*^9}},ExpressionUUID->"b488b4c6-c8e4-4649-8631-\
e67a60058eb2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU9", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU4", ",", "U1"}], "]"}], ",", "SO9", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "SU3"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "1215"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774563995188139*^9, 3.774564006668433*^9}, {
   3.7823308649903464`*^9, 3.782330865291547*^9}, 3.782330897302181*^9, 
   3.782330930382366*^9, 3.782331277228551*^9, {3.7823316429441185`*^9, 
   3.7823316436948986`*^9}},ExpressionUUID->"98007754-71ba-409d-b396-\
dd3cb47d8f36"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"4.9\"\>",
      4.8585765,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[4.8585765, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615255911079797*^9, 3.7823308909750953`*^9, 
  3.7823309259156857`*^9, 3.782330960294383*^9, 3.782331307961425*^9, 
  3.7823316528175526`*^9},ExpressionUUID->"18bbaf07-7eb4-48b1-892c-\
97257497b084"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "156", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(9)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"240", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"396", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-32)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["630",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["504",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1050", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["504",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1215", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["216", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"240", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"396", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["112", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1050", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1215", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"240", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"396", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1050", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1215", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"240", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"396", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1050", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1215", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(9)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["9",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["36",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["84",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["126",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["156",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"240", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"396", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["156",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["450",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["495",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["910",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1050", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1215", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["495",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"240", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"396", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1050", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1215", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6152559113416767`*^9, 3.782330891459832*^9, 
  3.7823309263545423`*^9, 3.7823309607751255`*^9, 3.782331308493*^9, 
  3.7823316532827206`*^9},ExpressionUUID->"30e2d29b-cf71-4c44-a941-\
090e2200f252"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU10", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU5", ",", "U1"}], "]"}], ",", "SU3", ",", "SU4", 
     ",", "SU5", ",", "Sp4", ",", "SO10", ",", "Sp10", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU2"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "1485"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7745640330957823`*^9, 3.774564075588131*^9}, {
  3.7836766535753098`*^9, 
  3.783676654452964*^9}},ExpressionUUID->"08b23165-591c-4e27-8ff3-\
9d2ba1de2783"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"11.2\"\>",
      11.1726173,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[11.1726173, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256085582604*^9, 3.777629169222994*^9, 
  3.7788381483825593`*^9, 3.7823318192795005`*^9, 
  3.783676688365305*^9},ExpressionUUID->"1e279120-92b0-4f04-bee6-\
e481867d5046"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "66", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(10)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["396",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-36)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-26)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["630",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-26)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["990",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["630",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(3)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["10",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["125",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["125",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["162",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["162",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["125",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["125",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["154", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["125",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["143",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["162",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(4)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["10",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["45",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["84",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["270",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["360",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["360",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["300",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["315",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["360",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(5)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["10",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox[
                OverscriptBox["45", "_"],
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["5", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["50", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["175", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["175",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["315",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["15", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["210", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["490", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["15", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["40", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["210", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["560", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["40", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["175", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["315", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["450", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["70", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["105", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["480", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["40", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["175", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["210", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1050", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(4)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["10",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["220",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["220",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["220",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(10)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["10",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["45",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["120",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["210",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["210",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["210",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["660",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["770",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["945",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["945",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1386",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(10)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["10",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["55",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["110",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["220",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["110",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["132",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["110",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["220",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["715",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["780",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["891",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["891",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1430",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1485", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.61525608586572*^9, 3.7776291698133993`*^9, 
  3.77883816869175*^9, 3.782331820194823*^9, 
  3.7836766891312675`*^9},ExpressionUUID->"40973258-04b3-4ee8-93df-\
b9f1bd8a133f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU11", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU10", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU5", ",", "U1"}], "]"}], ",", "SO11"}], "}"}], 
   ",", 
   RowBox[{"MaxDim", "\[Rule]", "1210"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7745641183964415`*^9, 3.774564122613164*^9}, {
   3.782332515117999*^9, 3.7823325156814923`*^9}, 3.78233262584829*^9, {
   3.782332776858176*^9, 3.7823327793096256`*^9}, 3.782332888687991*^9, 
   3.7823330481142073`*^9, {3.7837693943440247`*^9, 
   3.78376939527849*^9}},ExpressionUUID->"f0d228c9-a713-4ceb-92a7-\
e707d084d931"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"95.1\"\>",
      95.1427824,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[95.1427824, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256206060774*^9, 3.7776291427000413`*^9, 
  3.7823326067804213`*^9, 3.782332717225439*^9, 3.7823328700518427`*^9, 
  3.7823329781812987`*^9, 3.7823331408787003`*^9, 
  3.7837694918239665`*^9},ExpressionUUID->"c8b25173-035e-4b17-ae76-\
b55bc85d752b"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "134", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(11)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["825",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["396",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-36)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-32)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(11)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["11",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["55",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["165",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["275",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["330",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["429",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["462",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["429",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["275",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["429",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1144",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256206242445*^9, 3.777629143015158*^9, 
  3.7823326071055174`*^9, 3.7823327175016713`*^9, 3.7823328703290725`*^9, 
  3.7823329784595194`*^9, 3.7823331412318497`*^9, 
  3.783769492228879*^9},ExpressionUUID->"a566e394-03cf-4174-b7e8-\
5058b6a43466"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU12", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU11", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU10", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU6", ",", "U1"}], "]"}], ",", "SO12", ",", "Sp12",
      ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU3"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "1800"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.77456412915666*^9, 3.774564166849847*^9}, {
   3.7823378477310824`*^9, 3.7823378482786207`*^9}, 
   3.78234343528556*^9},ExpressionUUID->"2fc0d887-8500-4152-824c-\
6da41c8c9258"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"8.6\"\>",
      8.6015371,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[8.6015371, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152566638320503`*^9, 3.7776296473962736`*^9, 
  3.782338275008396*^9, 
  3.782343499249259*^9},ExpressionUUID->"af05d8c5-3f23-47a5-b048-\
8e7856e3b125"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "175", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(12)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-33)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["825",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["396",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-35)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(12)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["12",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["66",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["66",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["220",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["352",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["495",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["220",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["792",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["352",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["462",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["462", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1638",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(12)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["12",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["78",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["208",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["364",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["429",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["208",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["208",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["572",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["364",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["429",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["429",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1650",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256664018915*^9, 3.777629648314779*^9, 
  3.7823382753454857`*^9, 
  3.782343499480635*^9},ExpressionUUID->"7767be2d-9642-4664-8383-\
cd4fce1b26a4"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU13", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU12", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU11", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU10", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU6", ",", "U1"}], "]"}], ",", "SO13"}], "}"}], 
   ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "1170"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7746157040790615`*^9, 3.7746157051387053`*^9}, {
  3.78377242765221*^9, 
  3.78377243220201*^9}},ExpressionUUID->"14ed25d7-27d9-4a62-89e2-\
157d6448ed68"],

Cell[BoxData[
 FormBox[
  TemplateBox[{
   "NumberForm","sigz",
    "\"In addition to the number of digits requested, one or more zeros will \
appear as placeholders.\"",2,144,31,2.880405022486932*^19,"Local"},
   "MessageTemplate"], TraditionalForm]], "Message", "MSG",
 CellChangeTimes->{3.776687722181971*^9, 
  3.78377412334614*^9},ExpressionUUID->"f5788445-af66-435a-b65a-e63eafcb44fb"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"1690.0\"\>",
      1686.856603,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[1686.856603, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.776687722185931*^9, 
  3.7837741240161147`*^9},ExpressionUUID->"15cd0dba-9b53-4dc0-a847-\
8f0311c3f761"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "137", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(13)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(13)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"13", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-36)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(13)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"13", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-33)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-33)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(13)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"13", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(13)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"13", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-36)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["396",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(13)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"13", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-32)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(13)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"13", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(13)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(13)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"13", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["13",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["78",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["286",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["442",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox[
                OverscriptBox["715", "_"],
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["715",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["286",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["715",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["442",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["715",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.776688150428475*^9, 
  3.783791456162925*^9},ExpressionUUID->"83bb7774-47f5-4e84-b8b2-\
2bd37052b46c"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU14", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU13", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU12", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU11", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU10", ",", "SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "SU6", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU7", ",", "U1"}], "]"}], ",", "SO14", ",", "Sp14",
      ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU2"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "910"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7746157166348352`*^9, 3.7746157179533815`*^9}, {
  3.774615845682345*^9, 3.7746158466577935`*^9}, {3.7838872266009326`*^9, 
  3.783887227128505*^9}},ExpressionUUID->"6c8308e3-6a31-42a0-aad7-\
acb9243c19e7"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"201.0\"\>",
      201.4190376,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[201.4190376, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.776692052116666*^9, 
  3.7838874297966146`*^9},ExpressionUUID->"af581579-75ea-4af1-bd6e-\
67b671cb041b"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "172", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(14)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(13)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-26)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"195", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["13", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["728",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"195", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"195", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-33)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-33)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"195", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"195", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"195", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"195", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["14",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["91",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["104",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"195", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["104",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["364",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["896",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(14)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["14",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["105",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"195", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["896",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"195", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.776692053598274*^9, 
  3.783887430089796*^9},ExpressionUUID->"c8003458-0871-4707-b625-\
ba5b4cf25a33"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU15", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU14", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU13", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU12", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU11", ",", "SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU10", ",", "SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "SU6", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "SU7", ",", "U1"}], "]"}], ",", "SO15", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU3"}], "]"}], ",", "SU5", ",", "SU6"}], "}"}], 
   ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "1560"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7746156381175375`*^9, 3.774615644874525*^9}, {
  3.774615736530386*^9, 3.774615738487261*^9}, {3.7766923040458035`*^9, 
  3.77669230529834*^9}, {3.776693059432229*^9, 3.7766930598977633`*^9}, {
  3.7839576932443132`*^9, 3.7839576945797405`*^9}, {3.7839602685643983`*^9, 
  3.783960269308382*^9}},ExpressionUUID->"93caca86-03d2-465e-81ba-\
fa912fde7f2e"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"671.0\"\>",
      671.4548253,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[671.4548253, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7766929640659976`*^9, 3.7766931122256875`*^9, 
  3.7788388764025855`*^9, 3.783958373954191*^9, 
  3.783960941471944*^9},ExpressionUUID->"9048ccc1-ca73-4aac-bd59-\
f47a4858c16a"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "258", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(15)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["14", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1260",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(13)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-26)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-26)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["13", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-39)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["13", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-33)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-33)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-44)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-29)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-26)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(21)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(21)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-32)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(28)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(15)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["15",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["105",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["119",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["119",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["455",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["1365",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["455",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1105",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(5)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["15",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox[
                OverscriptBox["105", "_"],
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["50", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["200",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["175",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["280",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["420",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["420",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1050", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["160",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1215",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["15",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox[
                OverscriptBox["105", "_"],
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["15", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["896",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["315",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1050",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["384",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1050",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.776692964440915*^9, 3.7766931124830027`*^9, 
  3.7788390825541277`*^9, 3.7839583745067463`*^9, 
  3.783960941900796*^9},ExpressionUUID->"7e2626bc-22e4-44a6-9084-\
2cd664cec4a6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU15", ",", 
   RowBox[{"{", "SU3", "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "1560"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.776693313411718*^9, 3.7766933527301435`*^9}, {
  3.7788392275886564`*^9, 3.778839229782854*^9}, {3.7839588157117248`*^9, 
  3.783958816791837*^9}, {3.783961181232267*^9, 
  3.7839611829097557`*^9}},ExpressionUUID->"2ff77ce0-33f0-423c-b37a-\
b31a9130451d"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"2.5\"\>",
      2.5006106,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[2.5006106, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{{3.7766933161355605`*^9, 3.776693365735379*^9}, 
   3.77883930068114*^9, 3.7839588200591125`*^9, 
   3.7839611860424204`*^9},ExpressionUUID->"41e32968-409b-477e-a66a-\
10b205548309"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "260", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(15)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(3)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["15",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["3", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["15", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["21", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["24", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["42", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["15", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["24", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["125",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(15)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(3)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SuperscriptBox[
                StyleBox["15",
                 FontWeight->Bold], "\[Prime]"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["42", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["63", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["125",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"455", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["143",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["162",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1365", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["192",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["195",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["15",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["132",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["192",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["273",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{{3.7766933161654797`*^9, 3.7766933657682905`*^9}, 
   3.7788393007230544`*^9, 3.7839588201378913`*^9, 
   3.783961186138125*^9},ExpressionUUID->"73ab3a39-292a-4b94-abf3-\
388a526d9f4b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU16", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU15", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU14", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU13", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU12", ",", "SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU11", ",", "SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU10", ",", "SU6", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "SU7", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "SU8", ",", "U1"}], "]"}], ",", "SO16", ",", "Sp16",
      ",", "SO10", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU4"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "560"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.78367819084155*^9, 3.7836782413936367`*^9}, {
   3.783889833992751*^9, 3.7838900581905537`*^9}, 
   3.7838932694613233`*^9},ExpressionUUID->"1dd7acfa-8b79-4ecd-9165-\
186a77c45b3d"],

Cell[BoxData[
 FormBox[
  TemplateBox[{
   "NumberForm","sigz",
    "\"In addition to the number of digits requested, one or more zeros will \
appear as placeholders.\"",2,239,48,2.880405022486932*^19,"Local"},
   "MessageTemplate"], TraditionalForm]], "Message", "MSG",
 CellChangeTimes->{
  3.783895405464035*^9},ExpressionUUID->"da9ef50a-aa24-45d5-9a68-\
7472eb5ae7b5"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"2100.0\"\>",
      2104.3826491,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[2104.3826491, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7838901349717875`*^9, 
  3.7838954073106837`*^9},ExpressionUUID->"d74c7e75-f368-48dc-b19b-\
8e80c17ffcbd"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "177", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SU(16)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(15)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["14", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(13)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-26)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-26)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["13", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-39)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-33)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(21)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["16",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["120",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["135",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["135",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["560",
                FontWeight->Bold],
               TraditionalForm]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(16)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["16",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["119",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["136",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["119",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["136",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["544",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(10)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["16",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["120",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox[
                OverscriptBox["560", "_"],
                FontWeight->Bold],
               TraditionalForm]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"255", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.783890135190203*^9, 
  3.7838959944757338`*^9},ExpressionUUID->"4d07fa0d-17c4-4a28-953a-\
a589a3f609da"]
}, Open  ]],

Cell[CellGroupData[{

Cell["SO(N)", "Section",ExpressionUUID->"d894e233-7c41-4415-bb6d-75557baebd5a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO7", ",", 
   RowBox[{"{", 
    RowBox[{"SU4", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "U1"}], "]"}], ",", "G2"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "3080"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "8"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616224211698566*^9, 3.616224212094411*^9}, {
  3.774564473495332*^9, 3.77456450512271*^9}, {3.783882376418047*^9, 
  3.7838823788218713`*^9}},ExpressionUUID->"46655597-d117-4ba9-9c85-\
83eaa626a43b"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"16.1\"\>",
      16.1014322,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[16.1014322, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256718409889*^9, 3.616230142779251*^9, 
  3.7776345979763975`*^9, 
  3.7838823959601593`*^9},ExpressionUUID->"ba93dcec-303a-4000-9b5a-\
73d475bdd581"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "146", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox["\"SO(7)\"", TraditionalForm], "\" Branching Rules\""}, 
          "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"21", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"27", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"48", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"168", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"168", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"182", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"189", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"330", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"512", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["84",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"616", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"693", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"714", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"819", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"825", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["84",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["280",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1254", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["540",
                   FontWeight->Bold], "\[Prime]\[Prime]\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["84",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["189", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["270",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["270", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1512", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["360",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["360", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1617", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["84",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["360",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["360", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1750", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["735",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1911", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2016", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["280",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["504",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["504", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["270",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["270", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["540",
                   FontWeight->Bold], "\[Prime]\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["825",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2310", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["189", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["270",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["270", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["120",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["280",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["420",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["420", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["500",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["500", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["360",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["360", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["960",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["825",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["960",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3024", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["216", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["420", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["360",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["360", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["500",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["500", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"21", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"27", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"48", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"168", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"168", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"182", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"189", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"330", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"512", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"616", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"693", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"714", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"819", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"825", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1254", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1512", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1617", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1750", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1911", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2016", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2310", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3024", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"21", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"27", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"48", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"168", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"168", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"182", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"189", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"330", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"512", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"616", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"693", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"714", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"819", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["30",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"825", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1254", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(14)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-14)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["204",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1512", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1617", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1750", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(6)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-6)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["30",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["30",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["55",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["260",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1911", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["30",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["30",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2016", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["224",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["224",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(16)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-16)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(14)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-14)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["204",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["204",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["285",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2310", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["30",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["30",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["30",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3024", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["56",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["56",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["56",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["56",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox[
                SubscriptBox["\<\"G\"\>", "2"],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"21", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"27", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"48", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"168", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"168", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"182", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"189", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"330", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"512", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"616", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"693", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"714", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["714",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"819", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"825", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1254", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["1254",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["714",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1512", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1617", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1750", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1911", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2016", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["714",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                SuperscriptBox[
                 StyleBox["2079",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2310", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["714",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1254",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["748",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3024", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["714",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256718535226*^9, 3.616230142963236*^9, 
  3.7776345988231153`*^9, 
  3.7838823967590218`*^9},ExpressionUUID->"8399acee-938a-42f7-a364-\
a4d2115127c8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO8", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SU2", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "U1"}], "]"}], ",", "SU3", ",", "SO7", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SU2"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "3696"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "6"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774564530458948*^9, 3.774564597723045*^9}, {
  3.7776347040962353`*^9, 3.777634707588078*^9}, {3.7823440824323444`*^9, 
  3.7823440841986346`*^9}},ExpressionUUID->"6c2b9ec3-0da1-4c4a-af2d-\
f46729fc5e52"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"72.2\"\>",
      72.2425298,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[72.2425298, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256738487112*^9, 3.7776346868603497`*^9, 
  3.7776347323249297`*^9, 
  3.782344159136549*^9},ExpressionUUID->"2f0eb717-56c9-4cd8-9277-\
2f3d25e021e5"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "1", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox["\"SO(8)\"", TraditionalForm], "\" Branching Rules\""}, 
          "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"28", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"350", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"28", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"350", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["36",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["36", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["36",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["36", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["189", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["270",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["270", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["336",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["189", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["270",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["270", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["50",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["280",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["280", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["280",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["280", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["224",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["224", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["360",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["360", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["224",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["224", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["360",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["360", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["140",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["60",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["60", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["360",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["360", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["60",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["60", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["360",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["360", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["140",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["45",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["45", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["45",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["45", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["70",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["70", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["84",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["175",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["70",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["70", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["84",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["189", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["270",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["270", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(6)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-6)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["50",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["50",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["735",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["189", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["270",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["270", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["216", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["420",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["420", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["540", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["280",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["280",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["280", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["280", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["504",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["504", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["280",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["280",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["280", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["280", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["504",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["504", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["216", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["420",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["420", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["540", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["189", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["189", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["270",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["270",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["270", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["270", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["189", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["189", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["270",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["270",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["270", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["270", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"28", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"350", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"28", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                SuperscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                SuperscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"350", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["378",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                SuperscriptBox[
                 StyleBox["1386",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                SuperscriptBox[
                 StyleBox["1386",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["378",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["714",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["693",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["819",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1008",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1008",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1008",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["1008",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1008",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["1008",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["693",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["825",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1560",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1560",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1008",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1008",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1617",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1617",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1008",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1512",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1008",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1512",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1617",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1617",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["819",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1560",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2310",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2310",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["378",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["819",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1750",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["1386",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2310",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["1386",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2310",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3024",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["1008",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2016",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3024",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["1008",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2016",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"28", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"35", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"56", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"112", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"160", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"224", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"294", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"350", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"567", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"840", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1296", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1568", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1680", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2400", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2800", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3675", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["30",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["260",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vc", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cv", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "cs", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sc", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["224",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "vs", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SubsuperscriptBox["\[AlignmentMarker]", "sv", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["224",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256738557097*^9, 3.7776346878746843`*^9, 
  3.777634733189583*^9, 
  3.7823441608319874`*^9},ExpressionUUID->"2354a838-0dca-4106-aa88-\
67e6791d6c43"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO9", ",", 
   RowBox[{"{", 
    RowBox[{"SO8", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "U1"}], "]"}], ",", "SU2", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "5148"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "7"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7745646164000883`*^9, 3.7745646570693183`*^9}, {
   3.7838832360292797`*^9, 3.7838832367503443`*^9}, 
   3.7838835385060472`*^9},ExpressionUUID->"ece41d6b-c031-46bc-a96f-\
9fdcfb77a421"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"30.9\"\>",
      30.8855641,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[30.8855641, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256761113978*^9, 3.7776476580194864`*^9, 
  3.782345004573147*^9, 3.783883268124484*^9, 
  3.7838835729144077`*^9},ExpressionUUID->"1ceb3d66-1771-4371-84c5-\
0ea30716015a"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "151", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox["\"SO(9)\"", TraditionalForm], "\" Branching Rules\""}, 
          "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"36", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"44", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"84", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"156", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"231", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"432", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "c"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vs"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vc"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"594", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sc"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cs"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sc"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cs"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"910", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"924", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cv"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sv"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1122", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "v", "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1920", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vs"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vc"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "vs"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "vc"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1980", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v", "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2457", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1296",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2508", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "v", "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1386",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vs"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vc"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "s"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2574", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1400",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cv"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sv"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "cs"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "sc"],
                 TraditionalForm], "+", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v", "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1568",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3900", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cv"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sv"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "s", "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "c", "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1296",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1400",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1925",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4158", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "cs"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "sc"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v", "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4608", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sc"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cs"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "sv"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "cv"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1296",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1296",
                   FontWeight->Bold], "c"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vs"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vc"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1400",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1400",
                   FontWeight->Bold], "c"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5040", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sc"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cs"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "c"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1296",
                   FontWeight->Bold], "s"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1296",
                   FontWeight->Bold], "c"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5148", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "v", "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["1386",
                   FontWeight->Bold], "v"],
                 TraditionalForm], "+", 
                FormBox[
                 SubscriptBox[
                  StyleBox["2640",
                   FontWeight->Bold], "v"],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"36", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"44", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"84", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"156", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"231", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"432", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"594", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"910", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"924", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1122", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1920", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1980", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2457", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2508", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2574", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3900", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4158", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4608", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5040", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5148", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["204",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"36", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"44", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"84", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"156", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"231", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"432", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"594", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"910", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"924", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1122", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1920", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1980", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2457", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2508", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["336",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["4", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2574", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3900", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4158", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["84",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4608", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["36",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["36", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["36",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["36", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5040", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["36",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["36", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["36",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["36", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["84",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5148", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["336",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["540",
                    FontWeight->Bold], "\[Prime]\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(7)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"36", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"44", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"84", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"156", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"231", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"432", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"594", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"910", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"924", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1122", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["378",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1920", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1980", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2457", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2508", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["378",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["378",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["714",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["48",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["48",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2574", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["693",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["189",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["819",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3900", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["189",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["693",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["693",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["825",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4158", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["378",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4608", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["720",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["720",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5040", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["112",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["112",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5148", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(14)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-14)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["378",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["378",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["378",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["714",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["714",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1254",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"36", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"44", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"84", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"156", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"231", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"432", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"594", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"910", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"924", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1122", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1920", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1980", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2457", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2508", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2574", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3900", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4158", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4608", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"27", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5040", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"33", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"27", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5148", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"36", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"44", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"84", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"156", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"231", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"432", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"594", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"910", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"924", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1122", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1920", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1980", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2457", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2508", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2574", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                  
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3900", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"14", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4158", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4608", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"14", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"13", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"13", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"14", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"14", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"14", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5040", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"18", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"16", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"16", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"14", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5148", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.6152567614274406`*^9, 3.7776476587784843`*^9, 
  3.782345005230401*^9, 3.783883268540374*^9, 
  3.783883573240534*^9},ExpressionUUID->"c0c52fb4-9f14-40cf-b1f3-\
9d90b4f15ed3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO10", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SU4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "U1"}], "]"}], ",", "Sp4", ",", "SO9", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp4"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "4312"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "6"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774565781072449*^9, 3.774565850135498*^9}, {
  3.7823453948481836`*^9, 3.7823453963483963`*^9}, {3.782345675851616*^9, 
  3.7823456802844925`*^9}, {3.7838838958501816`*^9, 3.7838839036042824`*^9}, {
  3.783884002085309*^9, 
  3.783884003006816*^9}},ExpressionUUID->"404d29c9-b272-4289-a9e7-\
5b6f211e186a"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"23.3\"\>",
      23.305242,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[23.305242, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152567844236727`*^9, 3.7776484344743814`*^9, 
  3.7776858394579277`*^9, 3.782345421845255*^9, 3.7823457034156203`*^9, 
  3.7838840302429724`*^9},ExpressionUUID->"7ca12e1d-8e47-4690-8f4d-\
7ad923737649"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "155", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox["\"SO(10)\"", TraditionalForm], "\" Branching Rules\""}, 
          "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(5)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"45", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"54", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"120", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"144", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["50", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"660", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["70",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["70", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["200",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(15)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["175", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"770", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["50", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["200",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"945", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1050", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["175", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1200", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["45", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["200",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-13)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["50", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["175",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["45",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["45", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["50", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1782", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["126", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["315",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["315", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["450",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["450", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["70", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["160", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["200",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["224", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["280",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["450",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["480", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(20)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(16)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["70", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["175", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["280",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["490",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["700",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2970", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["45",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["45", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["50", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["480",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["480", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["45", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(3)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["50", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["175",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["175", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["200",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["450",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["480", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(14)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(6)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["45", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["50",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["175", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["315", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["480", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["720",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4125", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["50", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1024",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4290", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["210", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["560",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["560", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["875",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["875", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1000",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["70",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["70", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["280",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["280", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["450",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["450", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["480",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["480", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(4)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"45", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"54", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"120", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"144", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"660", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"770", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"945", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1050", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1200", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1782", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2970", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["4", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4125", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4290", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["336",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(8)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"45", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"54", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"120", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"144", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"660", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"770", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"945", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["28",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1050", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cv"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sv"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1200", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["56",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cv"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sv"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1782", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubsuperscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "v", "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubsuperscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "v", "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2970", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["160",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["56",
                    FontWeight->Bold], "s"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["56",
                    FontWeight->Bold], "c"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cv"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cv"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sv"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sv"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4125", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubsuperscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "v", "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4290", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubsuperscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "v", "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubsuperscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "v", "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["1386",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["28",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["28",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["160",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["1296",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"45", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"54", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"120", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"144", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"660", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"770", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"945", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1050", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1200", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1782", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["390",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["625",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2970", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["390",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4125", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["390",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4290", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["390",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["455",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["625",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["810",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["390",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"45", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"54", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"120", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"144", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"660", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"770", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"945", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1050", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1200", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1782", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1122",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                SuperscriptBox[
                 StyleBox["2772",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2970", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2560",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2772",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4125", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1980",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4290", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1122",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2508",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2457",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(7)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"45", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"54", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"120", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"144", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"660", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"770", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"945", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1050", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1200", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1782", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["378",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2970", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["48",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["48",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4125", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["294",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4290", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["378",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["714",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"45", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"54", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"120", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"126", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"144", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"210", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"660", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"672", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"720", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"770", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"945", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1050", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1200", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1386", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1782", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2640", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2772", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2970", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3696", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4125", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4290", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.6152567849698277`*^9, 3.777648434724718*^9, 
  3.7776858408961067`*^9, 3.7823454222741823`*^9, 3.7823457038375645`*^9, 
  3.7838840307632437`*^9},ExpressionUUID->"239fd7b1-41cc-42e4-b95d-\
2c5d29d81ac1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO11", ",", 
   RowBox[{"{", 
    RowBox[{"SO10", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO9", ",", "U1"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "4290"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615255967591*^9, 3.774615267793022*^9}, {
  3.774615339341338*^9, 3.774615401665866*^9}, {3.782346050603342*^9, 
  3.782346051287628*^9}, {3.7823461808067904`*^9, 3.782346181599757*^9}, {
  3.7823463710444274`*^9, 
  3.782346371231946*^9}},ExpressionUUID->"d7f23682-9c10-4061-bc1c-\
45577f44a779"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"14.4\"\>",
      14.3998104,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[14.3998104, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7765309194468513`*^9, 3.777648552899679*^9, 
  3.7823460675850315`*^9, 3.782346195436611*^9, 
  3.782346386259617*^9},ExpressionUUID->"e0227d2d-8839-4e31-9e01-\
e925b8f270e9"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "15", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(11)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(10)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["16", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"65", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["210",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["16", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["144", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["144",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"429", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"935", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["210",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["660",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1144", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["770",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["144", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["144",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["560", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["945",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1760", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["16", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["144", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["144",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["720", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["210",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1386",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2717", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["210",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["660",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1782",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["945",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1728",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["560", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1200", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1200",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["672", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1440",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1440", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4290", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1050",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1050", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1728",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"65", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"429", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"935", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1144", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1760", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vs"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vc"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2717", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "v", "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cv"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sv"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sc"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cs"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sc"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cs"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sc"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cs"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4290", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cv"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sv"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"65", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"429", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"935", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1144", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1760", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2717", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4290", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"65", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"429", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"935", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1144", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1760", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2717", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["378",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4290", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"65", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"429", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"935", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1144", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1760", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2717", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1122",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4290", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["11",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"65", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"429", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"935", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1144", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["34",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1760", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["34",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2717", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["38",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4290", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.7765309205080104`*^9, 3.777648553535*^9, 
  3.7823460681745243`*^9, 3.7823461960231066`*^9, 
  3.7823463870496006`*^9},ExpressionUUID->"d9355c55-0b59-4979-afe8-\
bae24fe2067e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO12", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO8"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SU2"}], "]"}], ",", "SO11", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SO7"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "11088"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615689952729*^9, 3.7746156910948524`*^9}, {
  3.782346635061435*^9, 
  3.7823466360888023`*^9}},ExpressionUUID->"d21cfb39-b7d1-42cc-a5e5-\
9e45789a20af"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"114.0\"\>",
      113.5818681,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[113.5818681, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.776531076542641*^9, 3.777654583111543*^9, 
  3.7823467504102774`*^9},ExpressionUUID->"f446333c-285b-4c2b-9509-\
5920a39278d0"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "16", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox["\"SO(12)\"", TraditionalForm], "\" Branching Rules\""}, 
          "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(6)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"32", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"66", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"220", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"462", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["315", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["405",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1638", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["405",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["210", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2860", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["210", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["315", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["405",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["252", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["700",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["700", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["1050",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["1050", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4224", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(9/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["490",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["490", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1176",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1176", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4752", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["420",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["420", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["840",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["840", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["84",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["336",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["336", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["840",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["840", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1050",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1050", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["84",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["840",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["840", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1260",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1260", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8085", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["105",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["189",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["210", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["896",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["896", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1050",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1050", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["189",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(3/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["189",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-3/2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["896",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["896", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1050",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1050", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1176",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1176", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9152", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["210", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["315", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["315", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["405",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["405",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["840", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1701",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1701", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9504", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["210", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["896",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["896", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1050",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1050", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1470",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1470", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["420",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["420", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["840",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["840", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["1050",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["1050", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1134",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1134", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1260",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1260", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["504",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["504", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["700",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["700", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["720",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["720", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["1050",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["1050", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1134",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1134", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1260",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1/2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1260", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1/2)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(8)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"32", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"66", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"220", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"462", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1638", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2860", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubsuperscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "v", "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4224", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4752", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sv"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cv"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["28",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cv"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sv"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["28",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8085", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["56",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cv"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sv"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9152", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9504", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["1400",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["1568",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(4)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"32", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"66", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"220", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"462", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1638", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2860", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4224", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4752", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8085", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9152", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9504", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(10)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"32", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"66", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"220", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"462", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["660",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1638", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["45",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["720", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["720",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2860", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["54",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["660",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["660",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1782",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4224", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["672",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["672", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1440", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1440",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4752", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1050",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1050", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1728",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["120",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1728",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1200", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1200",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["320",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2970",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8085", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["210",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1050",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1050", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1728",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1728",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1200",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1200", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1200",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1200", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1440", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1440",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9152", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["720",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["720", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["720",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["720", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["2640", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2640",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9504", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["672", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["672",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1200", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1200",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1440",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1440", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1440",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1440", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["54",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["54",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["320",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4410",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["54",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["54",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["660",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["660",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4608",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"Sp(6)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"32", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"66", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"220", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"462", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1638", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2860", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4224", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4752", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8085", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["70",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["70",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["70",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["90",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["189",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["126",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9152", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["56",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9504", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["126",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"32", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"66", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"220", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"462", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1638", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2860", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4224", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4752", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8085", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"14", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"16", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9152", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9504", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"12", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"11", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"32", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"66", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"220", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"462", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["462",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["935",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1638", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1408",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1760",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2860", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2025",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["935",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2717",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4224", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["4224",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4752", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4290",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3003",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1408",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5005",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8085", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3003",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4290",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5280",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9152", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1760",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7040",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9504", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4224",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5280",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2025",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7150",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["935",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2025",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7293",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(9)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"32", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"66", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"220", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"462", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["450",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1638", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["576",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2860", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["910",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["450",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1122",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4224", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["672",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4752", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1650",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8085", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["126",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["672",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9152", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["576",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["576",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1920",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9504", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["672",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["672",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["44",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["910",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2574",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["44",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["450",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["910",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["2772",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(7)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"32", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"66", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"220", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"352", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"462", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"495", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1638", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2112", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2860", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4004", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["378",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4224", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4752", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8008", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8085", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9152", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9504", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["693",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["819",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.776531078074542*^9, 3.777654583924369*^9, 
  3.7823467515693207`*^9},ExpressionUUID->"ede9bf87-0452-49b6-8cff-\
394fb2eabe88"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO13", ",", 
   RowBox[{"{", 
    RowBox[{"SO12", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO11", ",", "U1"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "16575"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7746152708748207`*^9, 
  3.774615277184304*^9}},ExpressionUUID->"bfc6706b-3d3c-430c-a296-\
4a4d64edd8c1"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"98.3\"\>",
      98.3217694,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[98.3217694, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.776531236595007*^9, 
  3.777654822018787*^9},ExpressionUUID->"9b001e4a-e0d2-4a95-a41a-\
3b363979dc73"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "44", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox["\"SO(13)\"", TraditionalForm], "\" Branching Rules\""}, 
          "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"64", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"90", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"286", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"442", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"715", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1716", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1287",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2275", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3927", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2860",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4160", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4992", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2112", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2112",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5733", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1287",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4004",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7722", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4928",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12285", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8008",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["4928",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["4928", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4928",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8085",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1287",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2860",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["11088",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16575", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2860",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11088",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(10)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"64", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"90", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"286", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"442", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"715", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1716", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["660",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2275", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3927", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4160", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4992", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["720",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["720", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5733", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["660",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1782",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7722", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1728",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12285", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2970",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1200", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1200",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1050",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1050", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1728",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["54",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["660",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4608",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16575", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["54",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4410",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(8)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"64", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"90", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"286", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"442", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"715", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1716", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2275", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3927", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4160", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4992", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vs"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "vc"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5733", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubsuperscriptBox[
                   StyleBox["672",
                    FontWeight->Bold], "v", "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7722", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cv"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sv"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12285", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sc"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cs"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["56",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cv"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sv"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["294",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["1568",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16575", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["1400",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(7)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"64", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"90", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"286", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"442", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"715", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1716", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2275", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3927", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4160", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4992", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5733", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["378",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7722", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12285", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["20", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["819",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16575", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["693",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(9)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"64", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"90", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"286", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"442", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"715", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1716", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["450",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2275", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3927", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["910",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4160", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4992", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["576",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["576",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5733", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["450",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1122",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7722", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["36",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12285", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["36",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1650",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["84",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["44",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["450",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["910",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["2772",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16575", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["44",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["910",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2574",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(11)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"64", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"90", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"286", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"442", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"715", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1716", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["935",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2275", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["55",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3927", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["65",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2025",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4160", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4992", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1760",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1760",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5733", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["935",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["935",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2717",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7722", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["165",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12285", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["429",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5005",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3520",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3520",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["330",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4290",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-4)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["65",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["65",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["275",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["935",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["935",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2025",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2025",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7293",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16575", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["65",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["65",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["429",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2025",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2025",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7150",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"64", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"90", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"286", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"442", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"715", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"768", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1287", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1716", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2275", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3927", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4160", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["34",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["38",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4992", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["34",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["38",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5733", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7722", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12285", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"40", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"40", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"43", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"41", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"42", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"38", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"37", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13312", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"27", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"41", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"44", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"46", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"50", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"49", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"47", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"46", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"42", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"37", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"35", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["34",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["38",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["46",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14300", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"39", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"43", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"48", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"49", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"53", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"49", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"50", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"46", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"43", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"38", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"35", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"38", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"41", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"43", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"45", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"44", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"44", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"42", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"40", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"37", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"27", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16575", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"27", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"35", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"39", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"45", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"46", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"50", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"49", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"50", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"47", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"46", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"42", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"39", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"31", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.776531237425786*^9, 
  3.7776548227468376`*^9},ExpressionUUID->"f291319c-9c02-462b-9206-\
64bb8ebe5700"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO14", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO10"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO8"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "U1"}], "]"}], ",", "Sp4", ",", "Sp6", ",", "G2", 
     ",", "SO13", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "SO7"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "8000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7745665563583903`*^9, 
  3.774566663445983*^9}},ExpressionUUID->"bef074f0-3ec6-4ac5-8dc5-\
a76ec69bb67d"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"64.6\"\>",
      64.6408407,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[64.6408407, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152568035231113`*^9, 
  3.7776550168784876`*^9},ExpressionUUID->"053d11e9-82ae-4ab6-9932-\
2271e38d993d"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "47", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(14)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["189", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["112", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["189", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["490", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["588",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["588",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["588", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["735",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["112", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["490",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["490", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["784",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["735",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["735", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["735",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["735",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["735", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["490", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["588",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["735", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1323",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["735",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["735", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["735",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["189", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["735", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["756", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["840",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1323",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["660",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vc"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "vs"],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1287",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2860",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2112", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(4)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["14",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["64",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["220",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["140",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["160",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["220",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["220",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["220",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["455",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["390",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["285",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["625",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["390",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["455",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["390",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["625",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["390",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["405",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["455",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["224",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["420",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["390",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["455",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["595",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["625",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["224",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["320",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["420",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["640",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(6)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["14",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["64",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["385",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["14",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["216",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["126",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["126",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["385",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["525",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["385",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1274",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["385",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["525",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["126",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["385",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["525",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1078",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["525",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["14",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1386",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["385",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2205",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["14",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1344",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1386",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox[
               SubscriptBox["\<\"G\"\>", "2"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["14",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["64",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["273",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["286",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["273",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["286",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["748",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["273",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["286",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["286",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["714",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["273",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["286",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["273",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["286",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1547",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["77",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["896",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(13)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["64",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["286",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["442",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["768",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["715",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["286",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["715", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["1716",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["715", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1287",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["442",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1729",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1287",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1716",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["715",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2275",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["286",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["715",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2925",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["768",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4160",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["442",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["715",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3927",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["768",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4992",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["935",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1408",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1760",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256803629281*^9, 
  3.777655018070282*^9},ExpressionUUID->"73f63d58-5c2d-41ec-9d3a-\
4b608d97289f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO15", ",", 
   RowBox[{"{", 
    RowBox[{"SO14", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO13", ",", "U1"}], "]"}], ",", "SU2", ",", "SU4", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp4"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "11520"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.7746148424164906`*^9, {3.774614876105338*^9, 3.7746149960509615`*^9}, {
   3.7746150587703376`*^9, 3.7746150598405323`*^9}, {3.774615111423201*^9, 
   3.77461511415303*^9}},ExpressionUUID->"faa7a6f0-fec0-469c-b43e-\
503332e3882d"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"68.8\"\>",
      68.8029961,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[68.8029961, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.776531457916911*^9, 
  3.7776551173676815`*^9},ExpressionUUID->"de4e8b2a-a32f-41f1-a4ba-\
852ecb661ebb"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "48", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox["\"SO(15)\"", TraditionalForm], "\" Branching Rules\""}, 
          "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"15", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"119", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"455", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"665", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["546",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["832",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["832", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3080",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3003",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5355", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4004",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6435", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1716",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1716", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3003",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6916", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["546",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5265",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11520", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["832",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["832", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4928", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4928",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(12)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"15", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"119", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["32", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"455", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"665", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["352",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["32", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["32", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["352",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["352", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["792",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1638",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["462", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["792",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5355", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2079",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6435", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["462", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["792",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["792",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6916", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["352",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2860",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11520", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["32", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["32", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["352",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["352", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["352",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["352", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1728", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1728",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(10)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"15", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"119", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"455", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"665", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5355", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6435", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6916", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11520", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(8)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(7)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"15", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"119", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"455", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"665", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5355", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6435", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6916", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SubscriptBox[
                    StyleBox["8",
                    FontWeight->Bold], "v"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["112",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["567",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11520", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["112",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(9)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"15", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"119", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"455", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"665", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5355", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6435", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6916", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["156",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["910",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11520", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(11)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"15", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"119", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"455", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"665", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1144",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5355", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6435", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6916", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2025",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11520", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(13)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"15", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"119", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"455", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"665", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["442",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2275",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5355", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["78",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2925",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6435", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6916", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["90",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["442",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["442",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3927",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11520", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4160",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"15", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"119", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"455", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"665", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5355", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6435", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6916", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11520", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"35", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"36", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"35", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"27", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"15", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"119", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"455", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"665", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["360",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["360", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5355", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["189", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6435", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["729",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6916", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["300",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["875",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["875", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11520", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["270",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["270", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["630",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["630", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"15", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"119", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"128", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"455", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"665", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1105", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1792", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4080", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5355", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6435", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6916", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["81",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11520", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.776531459533585*^9, 
  3.7776551178184433`*^9},ExpressionUUID->"cfc23f7a-a0cc-4c99-be84-\
0ab2a279321c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO16", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO12"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO10"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO8"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "U1"}], "]"}], ",", "SO9", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp8", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp4"}], "]"}], ",", "SO15", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "SO9"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "6435"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615222502782*^9, 3.7746152238600492`*^9}, {
  3.774615729618788*^9, 
  3.7746157308393555`*^9}},ExpressionUUID->"75b1ab77-69ea-404a-a41a-\
3eceec034529"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"43.4\"\>",
      43.4009895,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[43.4009895, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7765315392175665`*^9, 
  3.777655216284197*^9},ExpressionUUID->"33672dc2-423d-4653-ab3a-\
385f2fa0be72"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "49", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(16)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["216", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["168", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["216", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["420", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["168", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["216", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["504",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["504", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["216", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["504",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["504", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1344", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1232",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1512",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1764",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["546",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["832", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["832",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3080",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1716",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1716", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3003",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(9)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["16",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["432",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["432",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["768",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["495",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["432",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["576",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["495",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1650",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1980",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["156",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1650",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3900",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["288",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["792",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["825",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(15)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["128",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["119",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["455",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["119",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["665",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["119",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["455",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1365",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1792",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1365",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3003",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["119",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4080",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["6435",
                FontWeight->Bold],
               TraditionalForm]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["442",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1287",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1287",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1716",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"135", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1820", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4368", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5304", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.77653154047919*^9, 
  3.7776552170511565`*^9},ExpressionUUID->"2a6e7bf6-56dd-4969-8eb2-\
291d836fbf59"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO17", ",", 
   RowBox[{"{", 
    RowBox[{"SO16", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO15", ",", "U1"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "6783"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615206641334*^9, 
  3.774615213405216*^9}},ExpressionUUID->"82da776f-f6c6-48b3-84f7-\
a26f4c25ea1e"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"41.1\"\>",
      41.0631696,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[41.0631696, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.776531593800521*^9},ExpressionUUID->"6c37db35-39dc-4443-9573-\
0e586f85766b"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "6", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(17)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(17)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["135",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["128", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"680", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"952", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["135",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["800",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1615", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["135",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1344",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1820",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["128", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1920", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1920",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1820",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4368",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6783", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["135",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1344",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5304",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(17)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"680", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"952", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["546",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1615", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["832",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["832", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6783", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3080",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(17)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"680", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"952", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1615", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6783", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(17)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"680", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"952", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1615", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6783", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(17)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"680", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"952", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1615", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6783", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(17)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"680", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"952", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1615", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6783", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(17)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"680", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"952", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["442",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1615", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1287",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6783", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(17)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"680", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"952", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["665",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1615", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1792",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1792",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3003",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6783", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4080",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(17)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["17",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"680", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"952", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1615", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6783", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.7765315942213974`*^9},ExpressionUUID->"dd3cbd60-0df7-41b4-b2ac-\
1db6723a1116"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO18", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO12"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO10"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO16", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU4"}], "]"}], ",", "SO17", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO9", ",", "SO9"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "5000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774566774411309*^9, 
  3.7745668949516077`*^9}},ExpressionUUID->"a294a84d-88df-47d6-9c48-\
67b7e63813d5"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"57.3\"\>",
      57.3038549,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[57.3038549, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152568468496523`*^9, 
  3.777655299539977*^9},ExpressionUUID->"d33b8531-fe66-4ef0-a6eb-\
4fd4aba6749e"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "50", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(18)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["396",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["396", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["240", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["396",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["396", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1215",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["240", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["630",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1008",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1050", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["546",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["832",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["832", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["800",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1920", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(17)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["136",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["152",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["256",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["136",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["680",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["152",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["952",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["136",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["152",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1615",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["680",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2380",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4096",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["665",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1792",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["442",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256846893395*^9, 
  3.777655306844482*^9},ExpressionUUID->"3f9786c8-0138-49bd-9b05-\
08b488d722af"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO19", ",", 
   RowBox[{"{", 
    RowBox[{"SO18", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO16", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO17", ",", "U1"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "9216"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615434836046*^9, 3.774615435997013*^9}, {
  3.7746157521846285`*^9, 3.7746157533109875`*^9}, {3.783885748561704*^9, 
  3.7838857494154215`*^9}},ExpressionUUID->"10705a46-b241-4669-b8b9-\
c8fbd6516675"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"19.8\"\>",
      19.7917493,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[19.7917493, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.776531666477068*^9, 
  3.7838862385501833`*^9},ExpressionUUID->"2caeabee-6b62-43f9-87e3-\
a8510743901e"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "165", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(19)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["153",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["170",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["256", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"969", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["153",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["816",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1311", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["170",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1122",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2261", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["153",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["170",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1920",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["816",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3060",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["256", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["4352", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4352",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"969", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1311", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["800",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2261", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1920", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"969", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1311", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["546",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2261", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["832",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["832", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"969", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1311", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2261", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"969", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1311", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2261", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"969", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1311", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2261", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"969", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1311", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["442",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2261", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"969", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1311", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["665",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2261", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1792",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1792",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"969", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1311", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["952",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2261", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["19",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["34",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"969", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1311", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2261", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["62",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.776531695069109*^9, 
  3.7838862388942623`*^9},ExpressionUUID->"6204cdd1-73a4-4157-8dd2-\
241363250db2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO20", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU10", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO12"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO10"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO18", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp10", ",", "SU2"}], "]"}], ",", "SO19", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO9", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "Sp4"}], "]"}], ",", "SU4"}], "}"}], 
   ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "9728"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615762949266*^9, 3.774615764222352*^9}, {
  3.7838868314877615`*^9, 3.7838868326994934`*^9}, {3.7840627605265355`*^9, 
  3.7840627618260746`*^9}, {3.784063902485968*^9, 
  3.7840639034943724`*^9}},ExpressionUUID->"923d140b-321d-4629-b9ef-\
532d927551d8"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"29.6\"\>",
      29.6091267,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[29.6091267, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7765318981970196`*^9, 3.7838870626186376`*^9, 
  3.7840630254616632`*^9, 
  3.784063941978119*^9},ExpressionUUID->"70ccc799-581d-414b-b693-\
abc81a0ba103"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "4", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(20)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["440", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["220", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["330", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["440", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1155",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1155", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1925",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["990",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["990", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1155",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1155", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2310",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2310", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["800",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1920", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["546",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["832",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["832", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1122",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4352",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1408",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1408",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(19)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["171",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["512",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["171",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["969",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1311",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["171",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2261",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["969",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3876",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9216",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["952",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["665",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1792",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["442",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(4)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SuperscriptBox[
                StyleBox["20",
                 FontWeight->Bold], "\[Prime]"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"190", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"209", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["256", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["300",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["336",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["256", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["735",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4845", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["256", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["735",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["875",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["875", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["189", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["360",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["360", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["875",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["875", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1280", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.7765319132663755`*^9, 3.7838870674906006`*^9, 
  3.784063105353402*^9, 
  3.7840639424499035`*^9},ExpressionUUID->"e115786b-7f34-4d3c-80f9-\
4cacc7d7cb25"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO21", ",", 
   RowBox[{"{", 
    RowBox[{"SO20", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO18", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO16", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO19", ",", "U1"}], "]"}], ",", "SU2", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO7"}], "]"}], ",", "SO7", ",", "Sp6"}], "}"}], 
   ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "20349"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615890828471*^9, 3.7746158910479064`*^9}, {
   3.774615924666498*^9, 3.7746160033922343`*^9}, {3.774616080682665*^9, 
   3.774616118536097*^9}, {3.783935505945073*^9, 3.7839355093110385`*^9}, 
   3.7839356935821114`*^9},ExpressionUUID->"c25fd1a7-18de-4caa-836d-\
778e29c54c5e"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"132.0\"\>",
      131.8132681,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[131.8132681, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7765319784917307`*^9, 
  3.783935683377504*^9},ExpressionUUID->"86c5f5eb-acc9-4ea6-ad48-\
bfb78df3aff3"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "182", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(21)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(20)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["190",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["209",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["512", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["190",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1140",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["190",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["209",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2640",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1140",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4845",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["209",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2640",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13090",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4845",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15504",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8550",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8568",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5304",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4368",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3080",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1287",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4080",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3003",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6783",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6188",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(19)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10640",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11628",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["21",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["34",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["69",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["71",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["693",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(7)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["21",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"2", "(", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1617",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["819",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1386",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1617",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["693",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1617",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["1911",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2079",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4095",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["378",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["693",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["819",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["825",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["1560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1617",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["2310",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4550",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(6)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["21",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["126",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"2", "(", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["126",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["385",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["525",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3059", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["126",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["924",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5985", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["126",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["385",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["525",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["924",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2205",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15939", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["385",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["525",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["924",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["1078",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1274",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3072",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3900",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20349", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["462",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["525",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["924",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2205",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["2457",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3072",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3276",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.776531978886733*^9, 
  3.7839356847212524`*^9},ExpressionUUID->"3d8b5787-f813-4e27-8288-\
3175dfc2342c"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO22", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU11", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO10"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO20", ",", "U1"}], "]"}], ",", "SO21", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO19"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO9", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO11", ",", "SO11"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "19228"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7745670505581384`*^9, 3.774567067084464*^9}, {
  3.7745672743969235`*^9, 3.774567399974064*^9}, {3.7839360677025843`*^9, 
  3.7839360706586533`*^9}},ExpressionUUID->"67edb12f-825c-409d-84f2-\
8e80013cbbe5"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"79.3\"\>",
      79.3147651,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[79.3147651, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256899384322*^9, 3.777656503838656*^9, 
  3.7839361518964543`*^9},ExpressionUUID->"cac4f490-4641-4a72-b696-\
63e3ca7ce887"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "184", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(22)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["330", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["594", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["440", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["594", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["330", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1760",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1760", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2904",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2904",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4235",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4719",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4719", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8550",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5304",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3080",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(20)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4845",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13090",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(21)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["230",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["1024",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1330",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["210",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["230",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3059",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5985",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["230",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3059",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15939",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(19)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6783",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4080",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19228", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256899454019*^9, 3.7776565082907553`*^9, 
  3.7839361565131216`*^9},ExpressionUUID->"7cb5bdd9-0de5-418f-ae67-\
87509e81fad8"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO23", ",", 
   RowBox[{"{", 
    RowBox[{"SO22", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO20", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO18", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO16", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO19"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO21", ",", "U1"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "8855"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615442425396*^9, 3.7746154442246776`*^9}, {
  3.7746157852655525`*^9, 3.7746157878078756`*^9}, {3.7839365512948895`*^9, 
  3.7839365521416574`*^9}, {3.783936923817705*^9, 
  3.7839369246773996`*^9}},ExpressionUUID->"639363d3-e70b-4dde-9380-\
c49369f73a55"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"66.8\"\>",
      66.8211447,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[66.8211447, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7765320783350353`*^9, 3.783936676522027*^9, 
  3.783936992436254*^9},ExpressionUUID->"8f9ef471-0351-4e2b-a3c8-\
4f4313a472de"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "188", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(23)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["252",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1540",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["1024", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1024",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["252",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3520",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1540",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7315",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(20)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4845",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(19)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(21)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5985",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["23",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1771", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["67",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4025", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8855", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["67",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["71",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["73",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.77653211080844*^9, 3.7839366769538355`*^9, 
  3.7839369931693025`*^9},ExpressionUUID->"193e2026-9c6b-4795-8a4c-\
cef002a7f93c"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO24", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU12", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO20"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO12"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO22", ",", "U1"}], "]"}], ",", "SO23", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO21"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SO19"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO9", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO11", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO8"}], "]"}], ",", "SU5"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "10626"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615652802954*^9, 3.7746156547144775`*^9}, {
  3.774615797455331*^9, 
  3.7746157988945208`*^9}},ExpressionUUID->"f7fb7f6b-0166-49ea-b7be-\
b47e682afdbe"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"191.0\"\>",
      190.8995283,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[190.8995283, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.776532375978201*^9},ExpressionUUID->"cccfc549-e3b3-47a8-8c0c-\
c119b2935c04"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "11", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(24)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["78", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["220", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["780", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["220", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["792", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["572", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["780", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["924", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["495", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2574", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4212",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(20)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4845",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(22)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1024", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(23)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["253",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["275",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["253",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1771",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["2048",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["253",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["275",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4025",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1771",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8855",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(21)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["1024",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5985",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(19)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cs"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(5)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["24",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"299", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["200",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["200",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["224",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["224", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1024",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2048", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"2", "(", 
               FormBox[
                StyleBox["1024",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["175",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["1024",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["1050",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["1050", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10626", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["126",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["175",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["224",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["224", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["1050",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["1050", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1176",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1701",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1701", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.776532391659685*^9},ExpressionUUID->"cc2d6505-4e1c-47a2-a29d-\
697ead76ab6b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO25", ",", 
   RowBox[{"{", 
    RowBox[{"SO24", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO22", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO20", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO18", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO16", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO19"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO21"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO23", ",", "U1"}], "]"}], ",", "SU2", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp4"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "32175"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7746154500336103`*^9, 3.774615451721923*^9}, {
  3.774615802423256*^9, 3.7746158037547684`*^9}, {3.7839372375864253`*^9, 
  3.783937238797188*^9}},ExpressionUUID->"a7df1aa9-d7b0-415c-8e96-\
b2550879da39"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"298.0\"\>",
      298.1315457,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[298.1315457, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.776532736144759*^9, 
  3.783937541461029*^9},ExpressionUUID->"10a7263f-f0df-4693-b154-\
f0cb9c371488"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "189", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(25)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(24)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["276",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["299",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["276",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2024",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2048",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["2048", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["276",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["299",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4576",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2024",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10626",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["299",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4576",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27300",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(22)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1024", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19228",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(20)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4845",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13090",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8550",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5304",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3080",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4080",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6783",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(19)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(21)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5985",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15939",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(23)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8855",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23000",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["25",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["67",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["67",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["69",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["73",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["79",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["67",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["69",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["71",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["79",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["85",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"38", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"38", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"37", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["87",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["93",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(25)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"25", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.7765327366365175`*^9, 
  3.783937656859544*^9},ExpressionUUID->"2fa68fe2-9fb6-46a0-b8de-\
926f20ba9458"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO26", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU13", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO22"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO20"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO16", ",", "SO10"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO24", ",", "U1"}], "]"}], ",", "SO25", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO23"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SO21"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "SO19"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO9", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO11", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO13", ",", "SO13"}], "]"}], ",", "F4"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "700000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774567462285721*^9, 
  3.774567626310543*^9}},ExpressionUUID->"b3e02d00-2f37-4933-8327-\
2141014b6a14"],

Cell[BoxData[
 FormBox[
  TemplateBox[{
   "NumberForm","sigz",
    "\"In addition to the number of digits requested, one or more zeros will \
appear as placeholders.\"",2,77,7,2.876439140193368*^19,"Local"},
   "MessageTemplate"], TraditionalForm]], "Message", "MSG",
 CellChangeTimes->{
  3.7776707173177357`*^9},ExpressionUUID->"3e920223-3cd1-44a1-b482-\
d865fd1e53ea"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"4650.0\"\>",
      4652.4458818,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[4652.4458818, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615257966993093*^9, 
  3.7776707173656335`*^9},ExpressionUUID->"351a2abb-5b7c-455f-8227-\
409f17ce6817"]
}, Open  ]]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "74", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox["\"SO(26)\"", TraditionalForm], "\" Branching Rules\""}, 
          "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(13)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["13", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["78", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["13", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["286", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1001",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1001", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-13)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["13", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["286", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1287", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["13", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["728",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["728", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1001",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1001", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1170",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1170", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["78", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3640",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3640", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5915",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["78", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["91", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["168",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3003", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3640",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3640", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5915",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6930",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6930", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9295",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["9295", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["13", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["286", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1001",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1001", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1287", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9009",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["9009", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21294", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(15)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["13", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-13)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["286", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["728", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1001", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(11)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1287", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3640",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-9)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["8008", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["9009", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15015",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16016",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20592",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21021", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["78", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["168",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1716", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-12)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3640",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3640", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5915",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16016",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16016", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-8)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["52052",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["52052", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["75712",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["13", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["286", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["728",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["728", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1001",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["1001", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1170",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1170", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8008",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["8008", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9009",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["9009", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21294", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["25025",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["25025", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["38025",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["38025", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54600",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["54600", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["13", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["286", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["728",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["728", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1001",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["1001", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-2)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1170",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1170", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13013",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["13013", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21294", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30030",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["30030", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["38025",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["38025", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["54600",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["54600", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64064",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["64064", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["13", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["286", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1001",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1001", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1287", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-14)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1716", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(14)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9009",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["9009", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21021",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21021", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-10)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21294",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21294", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["91091",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["91091", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["182182",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["182182", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(22)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1024",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1024", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3520",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["22",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3520",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["26334",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["26334",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1024",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1024", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1024",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1024", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21504",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21504", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["26334",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["74613",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["231",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3520",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3520",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["26334",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["133056",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["22",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["231",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3520",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3520",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3520",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3520",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19228",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["26334",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["191268",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1540",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["26334",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["26334",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["74613",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["170544",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(20)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["512", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["209",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2640",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4845",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["209",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2640",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17765",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4845",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15504",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["512", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["512", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["9728", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9728",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4845",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15504",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["38760",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["209",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["209",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2640",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4845",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17765",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["80256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["209",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["209",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2640",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2640",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2640",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13090",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17765",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["117040",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["190",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1140",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4845",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4845",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15504",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["38760",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77520",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(8)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(18)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["170",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1920",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3060",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["170",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1920",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11475",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3060",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8568",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4352", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4352",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3060",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8568",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18564",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "cv"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["224",
                    FontWeight->Bold], "sv"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["170",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1920",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3060",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11475",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45696",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["170",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["160",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["170",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1920",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1920",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1920",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8550",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11475",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["67830",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["153",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "c"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "s"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["816",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["56",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3060",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8568",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["8",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18564",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["31824",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(16)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(10)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["128", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["135",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1344",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1820",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["135",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1344",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7020",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1820",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4368",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["128", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["128", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1920",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["16", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1920", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1820",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4368",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8008",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["135",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1344",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1728",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1820",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7020",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24192",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["135",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["135",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1344",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1344",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1344",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2970",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5304",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7020",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36720",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1820",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4368",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8008",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11440",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(12)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(14)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["32", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["64", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["104",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["896",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1001",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["104",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["896",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2079",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4004",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["792",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1001",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2002",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["32", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["64", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["352", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["352",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["64", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["32", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["832",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["832", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["462", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["792",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1001",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2002",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["104",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["896",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1001",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2079",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4004",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4928",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11648",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["104",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["104",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["896",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["896",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["896",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1638",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2079",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3080",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4004",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8008",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18200",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["462", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["792",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["792",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["91",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1001",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1716", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["66",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2002",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(24)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2024",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2048",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["2048", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["299",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["299",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4576",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2024",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2024",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10626",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["276",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["299",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2024",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2024",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4576",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4576",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["37674",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2024",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10626",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10626",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42504",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["2048", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2048",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["2048", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2048",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["47104",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["47104", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10626",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42504",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42504",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["134596",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2024",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2024",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2024",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4576",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10626",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10626",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["37674",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["37674",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210496",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["276",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["299",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["299",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2024",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4576",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4576",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4576",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["37674",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["37674",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["299000",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42504",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["134596",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["134596",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["346104",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SO(25)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["4096",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["12650",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44550",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["12650",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["53130",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["98304",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["53130",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["177100",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["12650",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44550",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260820",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["32175",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44550",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["368550",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["177100",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["480700",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(23)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1771",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4025",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1771",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8855",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1771",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4025",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["31625",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1771",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8855",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["33649",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2048",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2048",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45056",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1771",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8855",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["33649",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["100947",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1771",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1771",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1771",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4025",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8855",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["31625",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["168245",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["253",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["275",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1771",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4025",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4025",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4025",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23000",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["31625",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["240350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8855",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["33649",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["100947",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["245157",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(21)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["230",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3059",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5985",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["230",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3059",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21735",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5985",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20349",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1024",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1024",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20480",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5985",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20349",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["54264",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["230",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3059",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5985",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21735",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["104006",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["230",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["230",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3059",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3059",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3059",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15939",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21735",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["150535",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5985",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20349",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["54264",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["116280",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(7)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(19)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["969",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2261",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["969",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3876",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["969",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2261",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["969",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3876",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11628",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["969",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3876",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11628",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["969",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["969",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["969",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2261",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3876",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["61047",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["969",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2261",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2261",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2261",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10640",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14364",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["89775",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["171",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["969",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3876",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11628",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50388",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(9)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(17)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["680",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["152",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1615",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["680",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2380",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["152",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["680",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1615",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9044",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["680",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2380",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6188",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4096",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["680",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2380",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6188",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12376",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["152",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["680",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["680",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["680",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1615",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2380",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9044",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["33592",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["152",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["152",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["680",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1615",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1615",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1650",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1615",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6783",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9044",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50388",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["136",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["680",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2380",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6188",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12376",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["19448",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(11)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(15)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["455",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["119",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["455",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1365",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["119",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["455",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5355",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["455",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1365",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1792",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["455",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1365",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5005",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["119",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["455",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["455",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["455",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1365",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5355",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17017",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["119",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["455",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["429",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["119",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1144",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4080",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5005",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5355",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["26180",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["455",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1365",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3003",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5005",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6435",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SO(13)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SO(13)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2925",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2925",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["768",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2925",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2925",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7722",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7722",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2275",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2275",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2925",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2925",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12285",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12285",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["715", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1287",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1716",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox[
                SubscriptBox["\<\"F\"\>", "4"],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2600", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1274",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["4096",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5824", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14950", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["1053",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8424",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52325", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["273",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1053",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1274",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["4096",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["8424",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10829",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19278",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65780", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["1053",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2652",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8424",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10829",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17901",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19448",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"102400", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1274",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2652",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8424",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10829",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19278",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["34749",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"230230", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1274",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2652",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8424",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10829",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["12376",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17901",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19278",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["34749",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["106496",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"320320", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["273",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["324",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1053",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["1053",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1274",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["2652",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["4096",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8424",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10829",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["17901",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19278",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["19448",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29172",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["34749",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["106496",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"450450", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["273",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["324",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1053",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["1053",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1274",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["2652",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["4096",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8424",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10829",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["17901",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19278",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19448",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["29172",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["34749",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["106496",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["107406",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"657800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1274",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["4096",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["8424",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10829",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["17901",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19278",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["29172",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["34749",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["76076",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["106496",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["119119",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["160056",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.615257967628932*^9, 
  3.7776709787449303`*^9},ExpressionUUID->"ab55466c-c65b-407f-99d6-\
9ac89bd7af71"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO27", ",", 
   RowBox[{"{", 
    RowBox[{"SO26", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO24", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO22", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO20", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO18", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO16", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO19"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO21"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO23"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO25", ",", "U1"}], "]"}], ",", "SU2", ",", "SU3", ",", "SO7", 
     ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO9"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "61074"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.774616141452772*^9, {3.7746161789930024`*^9, 3.7746162690331364`*^9}, {
   3.774616320900271*^9, 
   3.7746163227347765`*^9}},ExpressionUUID->"d3c0b523-70bf-4900-8517-\
6546dd669391"],

Cell[BoxData[
 FormBox[
  TemplateBox[{
   "NumberForm","sigz",
    "\"In addition to the number of digits requested, one or more zeros will \
appear as placeholders.\"",2,15,1,2.8757623809025315`*^19,"Local"},
   "MessageTemplate"], TraditionalForm]], "Message", "MSG",
 CellChangeTimes->{
  3.776534082341333*^9},ExpressionUUID->"b94635a0-9912-49ed-8903-\
8475b29ca93f"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"1270.0\"\>",
      1272.8377752,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[1272.8377752, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.776534082344328*^9},ExpressionUUID->"f00ecfe8-6873-400c-aedf-\
f606b925a756"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "13", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(27)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(26)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["325",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["325",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2600",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["325",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5824",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4096",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["4096", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2600",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14950",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5824",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37674",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["325",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2600",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5824",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["52325",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(24)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2048", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10626",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["37674",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(22)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1024", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19228",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26334",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(20)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4845",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13090",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17765",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8550",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11475",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5304",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7020",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3080",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4004",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4080",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5355",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6783",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9044",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(19)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(21)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5985",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15939",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21735",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(23)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8855",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["31625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(25)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12650",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44550",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["27",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["67",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["69",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["73",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["73",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["68",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["72",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["74",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["76",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["82",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["86",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["92",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["83",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["87",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["89",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["93",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"38", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"41", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"43", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"44", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"44", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"43", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"41", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"38", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["95",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["97",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["101",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"41", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"40", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"46", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"53", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"51", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"57", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"54", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"59", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"56", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"60", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"56", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"59", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"55", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"57", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"52", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"53", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["95",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["97",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["99",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(3)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["27",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["125",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["55", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["80", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["216",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["162",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["162", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["260", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["55", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["143",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["143", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"40", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["91", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["143", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["405", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["440",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["440", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"46", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"45", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"45", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"60", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"47", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"47", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"45", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["143", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["270",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["270", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["405", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(7)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["27",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["825",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["1617",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1750",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1911",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"2", "(", 
               FormBox[
                StyleBox["4096",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1617",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1911",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3375",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7722",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["714",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1617",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["1750",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1911",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["2079",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["3003",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4095",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8568",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10395",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["825",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1617",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1750",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1911",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["2310",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4550",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6545",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7722",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10395",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["231",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"43848", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["910",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"61074", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["495",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["910",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["910",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.7765349012322683`*^9},ExpressionUUID->"32f15d98-ca71-45c0-9c3a-\
cf6fa357e175"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO28", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU14", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO24"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO22"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO20"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO16", ",", "SO12"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "SO14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO26", ",", "U1"}], "]"}], ",", "SO27", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO25"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SO23"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "SO21"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO9", ",", "SO19"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO11", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO13", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO7"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "70875"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.7746163754550467`*^9, {3.7746165603906937`*^9, 3.774616583291037*^9}, {
   3.7746166226888776`*^9, 
   3.774616761678611*^9}},ExpressionUUID->"f249b961-10d6-489d-abe4-\
5349249fda1e"],

Cell[BoxData[
 FormBox[
  TemplateBox[{
   "NumberForm","sigz",
    "\"In addition to the number of digits requested, one or more zeros will \
appear as placeholders.\"",2,13,7,2.8758640258109317`*^19,"Local"},
   "MessageTemplate"], TraditionalForm]], "Message", "MSG",
 CellChangeTimes->{
  3.7766956910068254`*^9},ExpressionUUID->"c757c345-a00c-4479-9837-\
41ad01697262"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"2270.0\"\>",
      2266.7645382,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[2266.7645382, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.7766956910088515`*^9},ExpressionUUID->"8a1e674c-a7c0-4b0c-92a9-\
32901c3738f7"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "11", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(28)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["14", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["14", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["364", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1260",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["14", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["910", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1260",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1456",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1456", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1001", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3003",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1/2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3003", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1/2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1001", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5005",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5005", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8085",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3185",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3185", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8085",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10829",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12544",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12544", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["195",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4095",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4095", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5005",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5005", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8085",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9360",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9360", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12544",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12544", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(24)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2048", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10626",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["37674",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(22)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1024", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19228",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26334",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(20)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4845",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13090",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17765",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5304",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7020",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8550",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11475",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5304",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7020",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3080",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3080",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4004",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4004",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4096", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14950",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["37674",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["325",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52325",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(27)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["351",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["377",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["351",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2925",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["351",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["377",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6525",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["8192",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2925",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17550",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["377",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6525",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43848",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["351",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2925",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6525",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["61074",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(25)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["4096",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12650",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44550",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(23)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2048",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8855",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["31625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(21)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["1024",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5985",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15939",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21735",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(19)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1144",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6783",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9044",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2275",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4080",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5355",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3276", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8192", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20475", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.776536640881694*^9, 
  3.7767046548038797`*^9},ExpressionUUID->"fe2e7939-b71e-40e5-b1d8-\
71786b49f046"]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.774616773104108*^9, 3.7746168773396883`*^9}, 
   3.777665053349305*^9},ExpressionUUID->"9127f300-1f57-4a46-827e-\
01627ae08258"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO29", ",", 
   RowBox[{"{", 
    RowBox[{"SO28", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO26", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO24", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO22", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO20", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO18", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO16", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO19"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO21"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO23"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO25"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO27", ",", "U1"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "1"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "81809"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.77767147019313*^9, 3.77767147143783*^9}, {
  3.7776820394828587`*^9, 3.7776820423781214`*^9}, {3.7776820944228716`*^9, 
  3.7776821831230373`*^9}, {3.777682312358226*^9, 
  3.777682316559016*^9}},ExpressionUUID->"0917a4e1-d8e8-40ff-aeb5-\
46ffd067d14e"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"364.0\"\>",
      364.1560243,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[364.1560243, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.7776826865232496`*^9},ExpressionUUID->"87e3f744-3feb-435a-be78-\
21ec255e9f59"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "112", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(29)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(28)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3276",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["405",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7280",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["8192", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8192",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3276",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20475",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3276",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70875",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4096", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14950",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(24)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2048", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10626",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["37674",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(22)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1024", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26334",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(20)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4845",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17765",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11475",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7020",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4004",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5355",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9044",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(19)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(21)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5985",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21735",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(23)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8855",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["31625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(25)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12650",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44550",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(27)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6525",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8192",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8192",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17550",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["351",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6525",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6525",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["61074",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"29", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["29",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"406", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3654", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["69",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["71",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["73",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["79",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8091", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["79",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["83",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["68",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["72",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["74",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["76",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["82",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["86",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["88",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["90",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["92",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["94",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["96",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["100",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["106",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"23751", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["95",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["97",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["101",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81809", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"45", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"44", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"53", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"51", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"59", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"57", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"64", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"61", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"67", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"64", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"69", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"65", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"69", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"65", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"68", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"63", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"65", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"60", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"61", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"55", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"55", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"49", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"49", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["95",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["97",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["101",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["103",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["107",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.7776855727018175`*^9},ExpressionUUID->"3c3347e9-66dc-44f7-af4b-\
93e04b2073a7"]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.7745676650580673`*^9, 3.7745676662741795`*^9}, 
   3.7776650596076007`*^9},ExpressionUUID->"7be0c200-8452-43c8-a16e-\
965473d3093f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO30", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU15", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO26"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO24"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO22"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO20"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "SO16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO28", ",", "U1"}], "]"}], ",", "SO29", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO27"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SO25"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "SO23"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO9", ",", "SO21"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO11", ",", "SO19"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO13", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO15", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO10"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "SU4"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "93960"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7834263135296817`*^9, 3.7834263495039997`*^9}, {
   3.783426382191288*^9, 3.783426532814269*^9}, {3.783517694515418*^9, 
   3.783517702690591*^9}, 
   3.7835236763438807`*^9},ExpressionUUID->"51a89c42-5027-473c-95c3-\
e6b95c948d98"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"459.0\"\>",
      458.7558108,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[458.7558108, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7835185098516307`*^9, 
  3.7835251519627695`*^9},ExpressionUUID->"4f54caa4-3a5f-4ad8-8bb5-\
3bdc2814cd95"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "8", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(30)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(15)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["455", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1785",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1785", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1365", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3003",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5005", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6435",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1365", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10800",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["224",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5460",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5460", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10800",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12376",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12376", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16575",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16575", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4096", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14950",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["52325",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(24)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2048", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10626",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["37674",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(22)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1024", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26334",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(20)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4845",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17765",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11475",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4004",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7020",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(28)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3276",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8192",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3276",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3276",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20475",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3276",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3276",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70875",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(29)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["406",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["406",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3654",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["406",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["434",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8091",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["16384",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3654",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23751",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["406",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3654",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8091",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81809",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(27)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6525",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["8192",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17550",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6525",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["61074",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(25)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["4096",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12650",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44550",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(23)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2048",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8855",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["31625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(21)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["1024",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5985",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21735",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(19)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9044",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5355",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5355",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["672", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1200", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1440", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["54",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["660",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["770",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["945",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"435", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16384", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["84",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27405", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.7835236346650476`*^9, 
  3.783525153311059*^9},ExpressionUUID->"62e29ab3-ace9-4f11-bbc3-\
a20ebbc3c4d4"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Algebra", "[", "B", "]"}], "[", "15", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"SO30", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO28", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO26", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO24", ",", "SO7"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO22", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO20", ",", "SO11"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO18", ",", "SO13"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO16", ",", "SO15"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO14", ",", "SO17"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SO19"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SO21"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SO23"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SO25"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SO27"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO29", ",", "U1"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "1"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "76384"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7834262239815254`*^9, 3.7834262244183865`*^9}, {
  3.7834262595884266`*^9, 3.7834262744039526`*^9}, {3.783426687010949*^9, 
  3.783426796473827*^9}, {3.78351771631468*^9, 
  3.7835177184898715`*^9}},ExpressionUUID->"cb9e564b-f127-462b-8797-\
dae0c7374f22"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"347.0\"\>",
      347.2981463,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[347.2981463, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.7835265985294795`*^9},ExpressionUUID->"a0d9fbb2-4a32-49d9-9e51-\
27e1bf9d63fd"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "11", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"SO(31)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(30)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["435",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["435",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4060",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["435",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["464",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8960",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4060",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27405",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox[
                 OverscriptBox["16384", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16384",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(28)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20475",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8192",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(26)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["325",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2600",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14950",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4096", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(24)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["299",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["276",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10626",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2048", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(22)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1024", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(20)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["209",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["190",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4845",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(18)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["170",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["153",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3060",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["256", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1615",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["680",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["64", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(19)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2261",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["969",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3876",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(21)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3059",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5985",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(23)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4025",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1771",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8855",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2048",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(25)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12650",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(27)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6525",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17550",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8192",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8192",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(29)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["406",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["406",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["406",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3654",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["406",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["406",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["434",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["434",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8091",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["406",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3654",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3654",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["23751",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16384",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16384",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(31)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["31",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["71",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["79",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["85",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9889", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["85",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["87",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["89",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"31465", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["95",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["97",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["99",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["101",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["103",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["109",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"32768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["95",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["97",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["101",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["103",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["107",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["109",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["111",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["115",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["121",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.7835265990140705`*^9},ExpressionUUID->"afe637af-f5f4-41b5-a0a8-\
f33fefb25788"]
}, Open  ]],

Cell["\<\
Sp(n)
\
\>", "Section",
 CellChangeTimes->{{3.774567686274232*^9, 
  3.774567765629717*^9}},ExpressionUUID->"fd994986-96a0-4c19-8d75-\
c75c958ad1ce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp4", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "U1"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "7"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "154"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774567763001069*^9, 3.7745677839815216`*^9}, 
   3.774567817993681*^9, {3.774615457493794*^9, 3.774615460134879*^9}, {
   3.776705217093931*^9, 3.7767052271107225`*^9}, 3.777671487687374*^9, {
   3.7776726703757114`*^9, 3.777672684857996*^9}, 3.777672733338977*^9, {
   3.777672765015092*^9, 3.777672772330536*^9}, {3.7776728242999578`*^9, 
   3.7776728246979246`*^9}, {3.782350658356517*^9, 3.782350659028722*^9}, {
   3.7839390806788015`*^9, 3.7839390915307617`*^9}, {3.7839392677037764`*^9, 
   3.783939267964053*^9}},ExpressionUUID->"2f9dc048-a72c-42af-ae67-\
8f0847c935d1"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"0.7\"\>",
      0.682377,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[0.682377, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.7767051943768682`*^9, 3.776705276079177*^9, 3.77767268596404*^9, {
   3.7776727665111275`*^9, 3.7776727787025013`*^9}, 3.7823506601995883`*^9, {
   3.783939082154828*^9, 3.7839390928203163`*^9}, 
   3.783939269660528*^9},ExpressionUUID->"804c59df-e5d5-42e3-9b78-\
ad3bf953b8c5"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "196", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(4)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(4)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"40", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"154", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(4)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"40", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"154", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(4)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["4",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["5",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"40", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"81", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"154", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.7767051946252975`*^9, 3.7767052764698725`*^9, 3.777672686043826*^9, {
   3.7776727667275496`*^9, 3.7776727812327595`*^9}, 3.782350660496794*^9, {
   3.7839390825887003`*^9, 3.7839390930746346`*^9}, 
   3.7839392699846506`*^9},ExpressionUUID->"2b2bbecb-6bb6-4922-a20d-\
931dd8ed9c33"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp6", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp4"}], "]"}], ",", "SU2", ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "7"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "1078"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615464808789*^9, 3.774615467317217*^9}, 
   3.776704924345155*^9, {3.782350901392192*^9, 3.7823509316321974`*^9}, {
   3.782351013340745*^9, 3.7823510137287035`*^9}, 3.783939630012182*^9, {
   3.783939762893711*^9, 3.7839397667164574`*^9}, {3.7839403144081545`*^9, 
   3.7839403177551775`*^9}},ExpressionUUID->"57f847d4-dd43-4f0c-829a-\
3cc511bd8fe4"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"4.2\"\>",
      4.1626847,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[4.1626847, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.7767052836965704`*^9, {3.7823509033230276`*^9, 3.7823509334164314`*^9}, 
   3.7823510160455437`*^9, 3.78393977000072*^9, 
   3.7839403228695107`*^9},ExpressionUUID->"feda3966-0fa8-4e72-8906-\
593311a64158"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "202", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(6)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"385", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"448", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"616", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1078", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"385", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"448", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"616", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1078", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["6",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"385", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"448", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"616", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1078", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"385", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"448", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"525", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"616", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1078", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.776705284046231*^9, {3.7823509036162424`*^9, 3.7823509337764645`*^9}, 
   3.782351016517247*^9, 3.783939770322817*^9, 
   3.7839403237362275`*^9},ExpressionUUID->"d2c38b9d-2f25-44e1-a048-\
b45e47dabdf0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp8", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp6"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp4"}], "]"}], ",", "SU2", ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SU2"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "6"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "1716"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7746154763611155`*^9, 3.7746154783389273`*^9}, {
  3.782351244612363*^9, 3.782351245068145*^9}, {3.783940877423721*^9, 
  3.7839408782405376`*^9}},ExpressionUUID->"d36d3bb9-a3e0-4079-9abc-\
2839a688bd31"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"5.8\"\>",
      5.8341533,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[5.8341533, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.77670529294353*^9, 3.7823512513643126`*^9, 
  3.783940885163004*^9},ExpressionUUID->"42c45aa2-9314-48c8-9a3f-\
dfa3659be49c"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "205", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(8)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"42", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"288", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"308", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1056", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["84",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1232", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["84",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["84",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["189", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["270",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["270", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"42", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"288", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"308", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1056", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1232", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"42", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"288", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"308", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1056", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1232", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["8",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"42", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"288", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"308", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1056", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1232", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"42", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"288", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"308", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1056", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1232", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.7767052934545*^9, 3.7823512519238157`*^9, 
  3.783940885840225*^9},ExpressionUUID->"c42cc0b8-1cab-4857-854d-\
08855aef8335"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp10", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp8"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp6"}], "]"}], ",", "SU2", ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp4"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "6"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "4212"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615481621439*^9, 
  3.774615483571345*^9}},ExpressionUUID->"214bf111-4a67-4678-9bac-\
6151ec31a511"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"14.2\"\>",
      14.2453688,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[14.2453688, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.776705318371741*^9},ExpressionUUID->"80e004f6-a9e1-4a71-8c6e-\
74ea573b8174"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "16", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(10)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"44", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"110", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"132", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"891", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["175",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["315",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["315", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["450",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["450", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2860", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["480",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["480", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1024",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4212", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["280",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["450", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["480",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["480", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"44", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"110", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"132", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"891", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["288",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["288",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["792",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2860", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["825",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4212", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1232",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"44", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"110", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"132", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"891", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2860", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4212", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["10",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"44", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"110", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"132", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"891", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["34",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2860", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["38",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4212", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["38",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"44", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"110", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"132", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"891", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1155", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1408", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2860", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4212", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.776705318949047*^9},ExpressionUUID->"a29b9e73-2caa-459e-ac80-\
458fba5e8761"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp12", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp10"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp8"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp6"}], "]"}], ",", "SU2", ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp4"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "6"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "11440"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7746154909422073`*^9, 3.7746154928221054`*^9}, {
   3.7823528482207785`*^9, 3.782352848532916*^9}, 3.782352901811472*^9, {
   3.7823529371024404`*^9, 3.782352937365738*^9}, 3.7823529734170103`*^9, 
   3.7823531840534096`*^9, {3.7823532680239515`*^9, 3.7823532688268003`*^9}, {
   3.7823538891471634`*^9, 3.7823538907069664`*^9}, {3.7839412326956353`*^9, 
   3.7839412364894905`*^9}, {3.7839419816311374`*^9, 
   3.7839419820350823`*^9}},ExpressionUUID->"9f1b3149-1e6c-45e4-acfa-\
e25e8db041a3"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"39.3\"\>",
      39.2765205,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[39.2765205, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.776705383162263*^9, 3.776705593691185*^9, 
  3.7823528929312115`*^9, 3.7823529306836205`*^9, 3.782352965884122*^9, 
  3.7823530023395905`*^9, 3.7823532418150187`*^9, 3.782353925399853*^9, 
  3.783426073186767*^9, 3.783426135966867*^9, 3.783941368101613*^9, 
  3.7839420237076497`*^9},ExpressionUUID->"423c4078-4195-4493-836d-\
f8020f66dc65"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "210", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox["\"Sp(12)\"", TraditionalForm], "\" Branching Rules\""}, 
          "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(6)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"U(1)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"208", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"364", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"572", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["315", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["405",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["405",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2002", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(0)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["210", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["280",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["280", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["315", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["405",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["336",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["336", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["840",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["840", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["252", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["700",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["700", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["1050",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["1050", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-7)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["420",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["420", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["840",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["840", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6006", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-6)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["105",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["105", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["210",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["210", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["384", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(4)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["896",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["896", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1050",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1050", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["840",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["840", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1260",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1260", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["84",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["84", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["336",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["336",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["336", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["336", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["540",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["720",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["720", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["840",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["840", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1134",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1134", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1260",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1260", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["56", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["120",
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["120", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")", "\[InvisibleSpace]", 
                  FormBox["\<\"(-1)\"\>",
                   TraditionalForm]}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["420",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["420", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(5)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["560", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["840",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["840", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["1050",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["1050", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1134",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1134", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1260",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(1)\"\>",
                  TraditionalForm]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["1260", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}]}
             },
             AutoDelete->False,
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"Sp(10)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"208", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"364", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"572", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["780",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2002", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["891",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["891",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["715",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2002",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1155",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6006", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["891",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1155",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1408",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["780",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["891",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2860",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["891",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4212",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["220",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["780",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4620",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"Sp(8)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"208", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"364", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"572", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["308",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2002", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["288",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["792",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["288",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6006", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["48",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["288",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["308",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["792",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["36",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1232",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["308",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"Sp(6)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"Sp(6)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"208", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"364", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"572", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2002", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6006", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"208", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"364", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"572", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2002", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["34",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["38",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["34",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["38",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["46",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["34",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["38",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6006", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"31", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"31", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["34",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["38",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"37", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"40", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"41", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"42", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"40", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"39", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"35", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"33", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["34",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["38",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["46",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"27", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"36", "(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"38", "(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"41", "(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"40", "(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"40", "(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"38", "(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"36", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["32",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["34",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["38",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["46",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(4)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"208", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"364", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"572", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2002", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["140",
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["140", "_"],
                    FontWeight->Bold], "\[Prime]\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6006", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["84",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["256",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["256", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["126",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["126", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["50",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["50",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["196",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["300",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["384",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"Sp(4)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"65", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"78", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"208", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"364", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"429", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"560", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"572", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1365", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1650", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2002", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2925", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4368", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4576", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6006", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7800", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11088", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"10", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11440", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"9", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["40",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.7767053841268516`*^9, 3.7767055944771805`*^9, 
  3.7823528941110587`*^9, 3.7823529313717613`*^9, 3.782352966563341*^9, 
  3.782353003028739*^9, 3.78235324227977*^9, 3.782353925775837*^9, 
  3.783426074341879*^9, 3.783426136378825*^9, 3.7839413702578506`*^9, 
  3.7839420240956097`*^9},ExpressionUUID->"9b068b71-0cb5-4912-a551-\
ad6a15870b4a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp14", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp12"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp10"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp8"}], "]"}], ",", "SU2", ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO7"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "10752"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615498621909*^9, 3.7746155002242064`*^9}, {
   3.7823539042927046`*^9, 3.7823539074742007`*^9}, 3.782353981782752*^9, {
   3.782354019489545*^9, 3.7823540405972633`*^9}, {3.782354166240858*^9, 
   3.7823541867601857`*^9}, {3.7839429368560514`*^9, 
   3.7839429415863976`*^9}},ExpressionUUID->"6ac01025-85f3-4a76-92f4-\
0c4ca3384553"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"22.4\"\>",
      22.4484509,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[22.4484509, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.776705625079604*^9, 
  3.7839429689642053`*^9},ExpressionUUID->"55643729-05f8-4dfa-8f95-\
6439454053b1"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "213", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(14)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["189", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["112", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["189", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["490",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["490", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1638", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["588",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["588", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["112", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["490",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["490", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["784",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["735",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3094", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["735",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["735", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["735",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3900", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["735",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["735", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5355", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["735",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["735", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["735",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["189", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1386", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2156",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2156", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["588",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["588", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["840",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["840", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1323",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1323", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2016",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2016", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["429",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1638", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["429",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3094", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3900", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5355", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["4368",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4368",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1638", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3094", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3900", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5355", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1408",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1638", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3094", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3900", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5355", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["792",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["288",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["14",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["24",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["34",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["34",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["34",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["38",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1638", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3094", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3900", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5355", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["62",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["66",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["52",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"90", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1638", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3094", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3900", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5355", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["378",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["693",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["819",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["693",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.7767056260011983`*^9, 
  3.783942969345187*^9},ExpressionUUID->"1bc8f8b6-5439-43ac-a7e4-\
c8bc7e5b192c"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp16", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp12"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp10"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp8", ",", "Sp8"}], "]"}], ",", "SU2", ",", "Sp4", ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO8", ",", "SU2"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "9044"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7746155075439444`*^9, 3.77461550894627*^9}, {
   3.774615823631913*^9, 3.7746158255319195`*^9}, {3.7823544033164263`*^9, 
   3.7823544038280263`*^9}, 3.782354488830455*^9, {3.782354577437927*^9, 
   3.782354578749421*^9}, {3.7839442791586905`*^9, 
   3.7839442797979774`*^9}},ExpressionUUID->"53872b3e-553e-4f36-b484-\
90501040599f"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"41.3\"\>",
      41.3168118,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[41.3168118, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7767060079165945`*^9, 3.782354475628832*^9, 
  3.7823545620054207`*^9, 3.7823546432196712`*^9, 
  3.7839443270456595`*^9},ExpressionUUID->"86b2b423-84c7-41ee-be35-\
afb38220d3c8"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "218", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(16)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"119", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"544", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["216", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["168", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["216", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["420", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3808", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["504",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["504", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1344", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["330", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["924", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1232",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4862", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1176",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1176", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1764",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1232",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1512",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1512", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2352",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6885", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["420", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["945", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7072", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["168", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1008",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1008", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["2352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["2352", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9044", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["630",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["630", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["924", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["945", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1232",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"119", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"544", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3808", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2380",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4862", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3094",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6885", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3900",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7072", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9044", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5355",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"119", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"544", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3808", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4862", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["429",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["429",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6885", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7072", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["429",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9044", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"119", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"544", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3808", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4862", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6885", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7072", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9044", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"119", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"544", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3808", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4862", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6885", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7072", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9044", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["16",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"119", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"544", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["26",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["34",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["38",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3808", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["52",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4862", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6885", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7072", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["58",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9044", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(4)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["16",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"119", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"544", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["140",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["140",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["160",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3808", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["324",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["390",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["625",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4862", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["390",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["455",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["390",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["390",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6885", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["390",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7072", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["224",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["256",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["420",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9044", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["390",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["405",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["455",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(16)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"119", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"136", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"544", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1344", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3808", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "cv"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["224",
                   FontWeight->Bold], "sv"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3876", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4862", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v", "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6188", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubsuperscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v", "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6885", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7072", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "cs"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["672",
                   FontWeight->Bold], "sc"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9044", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SubscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "v"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["294",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["567",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.776706012353571*^9, 3.7823544792790775`*^9, 
  3.7823545625489445`*^9, 3.7823546436455264`*^9, 
  3.783944327798647*^9},ExpressionUUID->"4fbda093-bc14-4093-9885-\
6eb9c8ca6dc0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp18", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp12"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp8", ",", "Sp10"}], "]"}], ",", "SU2", ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO9"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp6"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "14364"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7835279299092484`*^9, 3.7835279305697813`*^9}, {
   3.7835283766456175`*^9, 3.783528515591236*^9}, {3.7835285700091496`*^9, 
   3.7835285722473917`*^9}, 3.783760479501232*^9, {3.7837605258131576`*^9, 
   3.783760527287922*^9}, {3.783761792771189*^9, 
   3.783761794320897*^9}},ExpressionUUID->"0500c5e8-3122-4ec2-abe8-\
fb1b142870ad"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"37.9\"\>",
      37.8715182,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[37.8715182, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7837606053043265`*^9, 3.783760894466708*^9, 
  3.783761921034399*^9},ExpressionUUID->"ac2e8d72-5347-4322-a419-\
1a550bb083e9"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "118", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(18)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"798", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["396",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["396", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["240", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["396",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["396", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2907", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1215",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1050",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1050", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2700",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2700", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1215",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1944",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2079", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11305", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["630",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["630", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1215",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2079", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["990",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["990", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1440", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1944",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2079", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"798", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2907", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1700",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1700",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3808",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11305", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6885",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9044",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"798", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2907", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3094",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11305", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3900",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5355",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"798", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2907", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11305", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"798", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2907", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11305", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["18",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"798", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["32",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["38",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["38",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["52",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2907", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["62",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["66",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11305", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["67",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"798", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2907", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11305", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"152", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"171", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"798", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2907", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7752", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8568", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11305", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.7837606122204323`*^9, 3.783760894766862*^9, 
  3.78376192149512*^9},ExpressionUUID->"9a863d6e-c48e-42e7-b4b6-1ca492eca04d"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp20", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU10", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp8", ",", "Sp12"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp10", ",", "Sp10"}], "]"}], ",", "SU2", ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "SU2"}], "]"}], ",", "SU6"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "2640"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.783528586420682*^9, 3.78352865925904*^9}, 
   3.7837615685814395`*^9, {3.783761604677822*^9, 3.783761605522416*^9}, {
   3.783761902542748*^9, 3.783761906123816*^9}, {3.7837621411911592`*^9, 
   3.783762142555375*^9}},ExpressionUUID->"81a65493-59b0-44cc-8760-\
14fc9eeeff71"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"196.0\"\>",
      195.6118016,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[195.6118016, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7837618121104126`*^9, 3.7837619403684607`*^9, 
  3.7837623388656716`*^9},ExpressionUUID->"75f9aa7b-2c8d-4059-8b6e-\
501719aa4d4d"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "121", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(20)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["440", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["220", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["330", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["440", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["540", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["816",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["20",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["38",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["52",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["52",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["58",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["52",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(20)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["20",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["189",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["560", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["540",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["980",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["70", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["540",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1960",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.78376181246443*^9, 3.783761940703531*^9, 
  3.783762339081072*^9},ExpressionUUID->"b5540fa5-f67f-49f8-8f6d-\
2abcadc430b7"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp22", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU11", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp20"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp8", ",", "Sp14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp10", ",", "Sp12"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "24794"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7835286801901054`*^9, 3.783528719480318*^9}, {
   3.783762211283538*^9, 3.783762225425356*^9}, {3.783946159301719*^9, 
   3.7839461606420026`*^9}, 
   3.7839481822201843`*^9},ExpressionUUID->"298d0292-6135-431d-86fa-\
61ababa3e30a"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"43.2\"\>",
      43.1963295,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[43.1963295, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.783762406043517*^9, 3.7837635563762817`*^9, 
  3.7839462452970767`*^9, 
  3.7839482297812595`*^9},ExpressionUUID->"849f3a10-0ee2-499f-9c2c-\
62bd812124bb"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "226", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(22)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1518", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["594", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["440", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["594", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7084", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["330", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1760",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1760", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2904",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19250", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2904",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4235",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4719",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4719", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24794", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3465",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3465", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8470",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8470", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(20)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1518", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7084", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4655",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19250", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24794", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4655",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1518", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7084", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2907",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19250", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8568",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24794", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2907",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7752",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1518", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7084", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1700",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19250", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24794", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1700",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3808",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1518", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7084", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19250", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3094",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24794", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1518", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7084", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19250", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24794", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["22",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"253", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1518", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["52",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["58",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["58",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["62",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7084", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["67",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["69",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["73",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19250", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24794", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["68",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["72",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["74",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["76",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["82",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["86",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.7837624097711754`*^9, 3.783763556560768*^9, 
  3.783946245656094*^9, 
  3.7839482300305715`*^9},ExpressionUUID->"dfa1282a-551a-4fc3-b537-\
8df5deef259e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp24", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU12", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp22"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp20"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp8", ",", "Sp16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp10", ",", "Sp14"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp12", ",", "Sp12"}], "]"}], ",", "SU2", ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "Sp6"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp8"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "Sp4"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "27324"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.783528728281344*^9, 3.7835288352902927`*^9}, {
  3.7837622133547883`*^9, 
  3.783762234601555*^9}},ExpressionUUID->"ac5742b9-25e9-4edc-9eb9-\
9e1482d8aef0"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"238.0\"\>",
      238.4326677,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[238.4326677, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.7837643937061234`*^9},ExpressionUUID->"7d1bacf3-a33b-4283-9888-\
0f757528f6c2"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "125", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(24)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["78", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["220", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["780", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["572", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["780", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["924", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["495", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2574", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4212",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["78", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1716",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1716", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4212",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5940",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(22)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7084",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19250",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(20)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4655",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2907",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8568",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1700",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3094",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["24",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["54",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["58",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["66",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["68",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["71",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["83",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["89",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["160",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["308",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["308",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["315",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(24)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.7837644226274757`*^9},ExpressionUUID->"063e0009-4cd3-4135-a935-\
924fbf0098c2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp26", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU13", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp24"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp22"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp20"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp8", ",", "Sp18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp10", ",", "Sp16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp12", ",", "Sp14"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "37700"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7835288533368406`*^9, 3.783528904428239*^9}, {
   3.7837622143888845`*^9, 3.7837622442268257`*^9}, {3.783948205368898*^9, 
   3.783948206861785*^9}, 
   3.783948303678663*^9},ExpressionUUID->"052345f1-4173-4da1-b352-\
6b20a0baafbb"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"660.0\"\>",
      660.4881203,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[660.4881203, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7837646023247814`*^9, 3.783948268424257*^9, 
  3.783948976409444*^9},ExpressionUUID->"5639a6b3-2c6a-45e3-b2e7-\
f62c7fe5475f"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "228", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(26)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(26)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(13)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["13", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["78", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2574", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["286", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1001", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["13", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["728",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1001", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1170",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1170", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14625", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["715", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3640",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3640", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5915",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"37700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2366",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2366", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5915",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9295",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9295", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(26)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(24)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2574", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14625", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"37700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(26)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(22)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2574", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14625", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7084",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"37700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["19250",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(26)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(20)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2574", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14625", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4655",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"37700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(26)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2574", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14625", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2907",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"37700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["308",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8568",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(26)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2574", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14625", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1700",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"37700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(26)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2574", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14625", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"37700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3094",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(26)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["26",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"324", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["31",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2574", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["62",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["66",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["68",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["72",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["74",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14625", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["79",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["83",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["85",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["89",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"37700", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"39", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"40", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"41", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"41", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"39", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"37", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["97",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.7837646504699736`*^9, 3.783948268927883*^9, 
  3.78394903516498*^9},ExpressionUUID->"89f26112-3d25-4ce3-beff-5ef89febe9c9"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp28", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU14", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp26"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp24"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp22"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp8", ",", "Sp20"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp10", ",", "Sp18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp12", ",", "Sp16"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp14", ",", "Sp14"}], "]"}], ",", "SU2", ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO7", ",", "Sp4"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "1"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "95004"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7835289081779222`*^9, 3.7835289442387743`*^9}, {
   3.7835289783496065`*^9, 3.7835289904722347`*^9}, 3.783762217017588*^9, {
   3.7837622508205523`*^9, 3.7837622522176476`*^9}, {3.783949623097918*^9, 
   3.783949625313935*^9}, {3.783951908386262*^9, 
   3.783951910092715*^9}},ExpressionUUID->"63395ef2-283c-4e4e-92b5-\
d34c307ab7df"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"274.0\"\>",
      273.7175219,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[273.7175219, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.78376524886112*^9, 3.7837656862267847`*^9, 
  3.7839510591156626`*^9, 
  3.7839521846946664`*^9},ExpressionUUID->"bfdba4d4-8893-45f3-8f0e-\
c6e98b24367f"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "234", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(28)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["14", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["364",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["364", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1260",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["14", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["910", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1260",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1456",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1456", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20097", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1001",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1001", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5005",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5005", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8085",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70470", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4095",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4095", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5005",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5005", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8085",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9360",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9360", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12544",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12544", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"95004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["2002", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13650",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["13650", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["31850",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["31850", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(26)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20097", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70470", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["51975",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"95004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["63180",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(24)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20097", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70470", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["37375",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"95004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40480",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(22)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20097", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7084",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70470", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26082",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"95004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7084",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24794",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(20)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20097", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4655",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70470", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17556",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"95004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4655",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20097", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2907",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70470", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11305",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"95004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2907",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7752",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20097", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1700",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70470", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6885",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"95004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1700",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3808",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20097", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70470", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3900",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3900",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"95004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["28",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["62",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["66",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["68",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["72",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["76",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["68",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["72",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["74",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["76",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20097", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["87",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["93",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["97",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70470", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"43", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"41", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"50", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"56", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"53", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"60", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"57", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"63", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"59", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"64", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"60", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"64", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"59", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"62", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"57", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"59", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"53", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"54", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["95",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["97",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["101",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["103",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"95004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"39", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"44", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"50", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"54", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"60", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"62", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"67", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"69", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"72", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"73", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"76", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"75", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"77", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"76", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"76", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"74", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"74", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"71", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"70", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"67", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"65", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"61", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"60", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"55", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"53", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"49", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"46", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"40", "(", 
                FormBox[
                 StyleBox["68",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["72",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["74",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["76",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["82",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["86",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["88",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["92",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["94",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["96",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["98",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["100",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["102",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["106",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["108",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["110",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["112",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["116",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(28)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"377", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20097", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70470", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["168",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"95004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["616",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["693",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["693",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["819",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.783765413600956*^9, 3.783765686663607*^9, 
  3.7839510646039667`*^9, 
  3.7839521851395016`*^9},ExpressionUUID->"3d2cabd6-3ef0-4dae-b621-\
ee4c20d5ae56"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"Sp30", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU15", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp28"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp26"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp6", ",", "Sp24"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp8", ",", "Sp22"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp10", ",", "Sp20"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp12", ",", "Sp18"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp14", ",", "Sp16"}], "]"}], ",", "SU2", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp10"}], "]"}], ",", " ", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"Sp4", ",", "Sp6"}], "]"}]}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "1"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "138446"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.783529013340949*^9, 3.783529080962178*^9}, 
   3.7837622191496725`*^9, {3.7837622595912113`*^9, 
   3.7837622612775097`*^9}},ExpressionUUID->"9fcc97c1-a9d5-45f7-bec6-\
d2f34fcf7827"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"787.0\"\>",
      787.1081887,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[787.1081887, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{
  3.7837668445373306`*^9},ExpressionUUID->"685c1148-7683-4467-a264-\
3084a317ae43"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "132", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox["\"Sp(30)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(15)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["455", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1785",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1785", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1365",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1365", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10800",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5460",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5460", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10800",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12376",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["12376", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16575",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16575", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3003",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3003", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20020",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20020", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["46200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["46200", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(28)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3248",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["406",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7280",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3248",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20097",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["377",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["406",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3248",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7280",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70470",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3248",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20097",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["95004",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(26)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5824",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["51975",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["63180",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(24)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4576",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["37375",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["275",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2000",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40480",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(22)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7084",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["253",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3520",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26082",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["230",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1518",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7084",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24794",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(20)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4655",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2640",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["17556",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4655",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14364",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(18)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2907",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["171",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2002",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11305",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["208",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["152",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["572",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["798",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2907",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7752",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(14)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1700",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["136",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["896",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3900",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6885",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["119",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["544",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1700",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3808",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["30",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["25",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["33",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["68",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["72",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["74",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["76",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["82",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["68",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["72",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["74",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["76",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["82",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["86",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["95",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["97",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["99",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["101",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"37", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"47", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"45", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"55", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"53", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"62", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"59", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"67", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"64", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"71", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"67", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"73", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"69", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"74", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"69", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"73", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"68", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"71", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"65", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"67", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"61", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"62", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"55", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"55", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"49", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"49", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["95",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["97",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["101",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["103",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["107",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["109",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["111",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"38", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"45", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"53", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"58", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"65", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"70", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"76", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"79", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"85", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"86", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"91", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"92", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"94", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"95", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"97", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"95", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"96", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"94", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"94", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"91", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"90", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"86", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"85", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"81", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"78", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"74", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"72", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"66", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"64", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"59", "(", 
                FormBox[
                 StyleBox["68",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"56", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"51", "(", 
                FormBox[
                 StyleBox["72",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["74",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"43", "(", 
                FormBox[
                 StyleBox["76",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"41", "(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["82",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["86",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["88",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["92",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["94",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["96",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["98",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["100",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["102",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["104",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["106",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["108",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["114",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["116",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["118",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["122",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["44",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["715",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["780",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["780",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["891",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["891",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["891",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["891",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1430",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1430",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["110",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["110",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["320",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1408",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1408",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1408",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1408",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2860",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2860",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2860",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4212",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4212",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4620",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"Sp(30)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"Sp(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"434", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4030", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8960", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"93496", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["35",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"138446", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["30",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.7837674234872913`*^9},ExpressionUUID->"d7b3115e-1c96-4d02-a9e4-\
c72c2576d44b"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exceptional Algebras", "Section",ExpressionUUID->"782ac299-fd42-47f3-b652-194f7304b238"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"E6", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "SU3", ",", "SU3"}], "]"}], ",", "F4", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "G2"}], "]"}], ",", "Sp8", ",", "G2", ",", "SU3"}], 
    "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "19305"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.77767362459501*^9, 3.7776736707391787`*^9}, {
   3.782355150633707*^9, 3.782355150987726*^9}, {3.7823553612295403`*^9, 
   3.78235536167336*^9}, 3.782355425139488*^9, 3.78235547505709*^9, 
   3.7823555367016954`*^9, 3.7823556020059175`*^9, {3.7823556594471655`*^9, 
   3.7823556600455704`*^9}, 3.7823557111169186`*^9, {3.783952324435048*^9, 
   3.7839523258033857`*^9}, {3.7839532076304398`*^9, 
   3.783953210383068*^9}},ExpressionUUID->"91367ffd-1b0b-4bf4-bbe8-\
26ec6b0f6e34"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"45.8\"\>",
      45.7521852,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[45.7521852, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615258043599415*^9, 3.777673724043145*^9, 
  3.7823552215301533`*^9, 3.7823553999475584`*^9, 3.782355464333338*^9, 
  3.7823555144081564`*^9, 3.782355574800841*^9, 3.782355639969239*^9, 
  3.782355698461587*^9, 3.782355806852337*^9, 3.783952465290903*^9, 
  3.7839532571986313`*^9},ExpressionUUID->"f6ad40c0-403c-4cb1-aea3-\
8f6237f0866b"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   TagBox[
    FrameBox[GridBox[{
       {
        ItemBox[
         TagBox[
          FormBox[
           TagBox[GridBox[{
              {
               TemplateBox[{StyleBox[
                  
                  TemplateBox[{"\"Table\[MediumSpace]\"", "238", "\": \""}, 
                   "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
                   FormBox[
                    SubscriptBox["\"E\"", "6"], TraditionalForm], 
                   "\" Branching Rules\""}, "RowDefault"]},
                "RowDefault"]},
              {
               TagBox[GridBox[{
                  {
                   TagBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton"],
                    "Grid"]}
                 },
                 AutoDelete->False,
                 
                 GridBoxAlignment->{
                  "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
                 GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
                 GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
                "Grid"]},
              {
               TagBox[GridBox[{
                  {
                   TagBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton"],
                    "Grid"]}
                 },
                 AutoDelete->False,
                 
                 GridBoxAlignment->{
                  "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
                 GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
                 GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
                "Grid"]},
              {
               TagBox[GridBox[{
                  {
                   TagBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton"],
                    "Grid"]}
                 },
                 AutoDelete->False,
                 GridBoxAlignment->{
                  "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
                 GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
                 GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
                "Grid"]},
              {
               TagBox[GridBox[{
                  {
                   TagBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton"],
                    "Grid"]}
                 },
                 AutoDelete->False,
                 
                 GridBoxAlignment->{
                  "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
                 GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
                 GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
                "Grid"]},
              {
               TagBox[GridBox[{
                  {
                   TagBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton"],
                    "Grid"]}
                 },
                 AutoDelete->False,
                 
                 GridBoxAlignment->{
                  "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
                 GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
                 GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
                "Grid"]},
              {
               TagBox[GridBox[{
                  {
                   TagBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton"],
                    "Grid"]}
                 },
                 AutoDelete->False,
                 
                 GridBoxAlignment->{
                  "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
                 GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
                 GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
                "Grid"]},
              {
               TagBox[GridBox[{
                  {
                   TagBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton"],
                    "Grid"]}
                 },
                 AutoDelete->False,
                 
                 GridBoxAlignment->{
                  "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
                 GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
                 GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
                "Grid"]},
              {
               TagBox[GridBox[{
                  {
                   TagBox[
                    TemplateBox[{"1"},
                    "OutputSizeLimit`Skeleton"],
                    "Grid"]}
                 },
                 AutoDelete->False,
                 
                 GridBoxAlignment->{
                  "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
                 GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
                 GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
                "Grid"]}
             },
             DefaultBaseStyle->"Column",
             
             GridBoxAlignment->{
              "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
             GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
             
             GridBoxSpacings->{
              "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
            "Column"],
           TraditionalForm],
          Short[#, 5]& ],
         BaseStyle->{Deployed -> False},
         StripOnInput->False]},
       {GridBox[{
          {
           TagBox[
            TooltipBox[
             StyleBox[
              StyleBox[
               DynamicBox[ToBoxes[
                 FEPrivate`FrontEndResource[
                 "FEStrings", "sizeBriefExplanation"], TraditionalForm],
                ImageSizeCache->{65., {3., 9.}}],
               StripOnInput->False,
               DynamicUpdating->True], "OSLText",
              StripOnInput->False],
             StyleBox[
              DynamicBox[
               ToBoxes[
                FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"], 
                TraditionalForm]], DynamicUpdating -> True, StripOnInput -> 
              False]],
            Annotation[#, 
             Style[
              Dynamic[
               FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"]], 
              DynamicUpdating -> True], "Tooltip"]& ], 
           ButtonBox[
            PaneSelectorBox[{False->
             StyleBox[
              StyleBox[
               DynamicBox[ToBoxes[
                 FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
                 TraditionalForm],
                ImageSizeCache->{52., {1., 9.}}],
               StripOnInput->False,
               DynamicUpdating->True], "OSLControl",
              StripOnInput->False], True->
             StyleBox[
              StyleBox[
               DynamicBox[ToBoxes[
                 FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
                 TraditionalForm]],
               StripOnInput->False,
               DynamicUpdating->True], "OSLControlActive",
              StripOnInput->False]}, Dynamic[
              CurrentValue["MouseOver"]],
             Alignment->Center,
             FrameMargins->0,
             ImageSize->{Automatic, 25}],
            Appearance->None,
            BaselinePosition->Baseline,
            
            ButtonFunction:>OutputSizeLimit`ButtonFunction[
             OutputSizeLimit`Defer, 301, 28804050224869316589, 5/2],
            Enabled->True,
            Evaluator->Automatic,
            Method->"Queued"], 
           ButtonBox[
            PaneSelectorBox[{False->
             StyleBox[
              StyleBox[
               DynamicBox[ToBoxes[
                 FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
                 TraditionalForm],
                ImageSizeCache->{62., {1., 9.}}],
               StripOnInput->False,
               DynamicUpdating->True], "OSLControl",
              StripOnInput->False], True->
             StyleBox[
              StyleBox[
               DynamicBox[ToBoxes[
                 FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
                 TraditionalForm]],
               StripOnInput->False,
               DynamicUpdating->True], "OSLControlActive",
              StripOnInput->False]}, Dynamic[
              CurrentValue["MouseOver"]],
             Alignment->Center,
             FrameMargins->0,
             ImageSize->{Automatic, 25}],
            Appearance->None,
            BaselinePosition->Baseline,
            
            ButtonFunction:>OutputSizeLimit`ButtonFunction[
             OutputSizeLimit`Defer, 301, 28804050224869316589, 5 2],
            Enabled->True,
            Evaluator->Automatic,
            Method->"Queued"], 
           ButtonBox[
            PaneSelectorBox[{False->
             StyleBox[
              StyleBox[
               DynamicBox[ToBoxes[
                 FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
                 TraditionalForm],
                ImageSizeCache->{44., {1., 9.}}],
               StripOnInput->False,
               DynamicUpdating->True], "OSLControl",
              StripOnInput->False], True->
             StyleBox[
              StyleBox[
               DynamicBox[ToBoxes[
                 FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
                 TraditionalForm]],
               StripOnInput->False,
               DynamicUpdating->True], "OSLControlActive",
              StripOnInput->False]}, Dynamic[
              CurrentValue["MouseOver"]],
             Alignment->Center,
             FrameMargins->0,
             ImageSize->{Automatic, 25}],
            Appearance->None,
            BaselinePosition->Baseline,
            
            ButtonFunction:>OutputSizeLimit`ButtonFunction[
             OutputSizeLimit`Defer, 301, 28804050224869316589, Infinity],
            Enabled->True,
            Evaluator->Automatic,
            Method->"Queued"], 
           ButtonBox[
            PaneSelectorBox[{False->
             StyleBox[
              StyleBox[
               DynamicBox[ToBoxes[
                 FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
                 TraditionalForm],
                ImageSizeCache->{78., {1., 9.}}],
               StripOnInput->False,
               DynamicUpdating->True], "OSLControl",
              StripOnInput->False], True->
             StyleBox[
              StyleBox[
               DynamicBox[ToBoxes[
                 FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
                 TraditionalForm]],
               StripOnInput->False,
               DynamicUpdating->True], "OSLControlActive",
              StripOnInput->False]}, Dynamic[
              CurrentValue["MouseOver"]],
             Alignment->Center,
             FrameMargins->0,
             ImageSize->{Automatic, 25}],
            Appearance->None,
            BaselinePosition->Baseline,
            ButtonFunction:>FrontEndExecute[{
               FrontEnd`SetOptions[
               FrontEnd`$FrontEnd, 
                FrontEnd`PreferencesSettings -> {"Page" -> "Advanced"}], 
               FrontEnd`FrontEndToken["PreferencesDialog"]}],
            Evaluator->None,
            Method->"Preemptive"]}
         },
         AutoDelete->False,
         FrameStyle->GrayLevel[0.85],
         GridBoxDividers->{"Columns" -> {False, {True}}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
         GridBoxSpacings->{"Columns" -> {{2}}}]}
      },
      DefaultBaseStyle->"Column",
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxDividers->{
       "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{
       "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.5599999999999999]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], 
          Offset[1.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}],
     BaseStyle->"OutputSizeLimit",
     FrameMargins->{{12, 12}, {0, 15}},
     FrameStyle->GrayLevel[0.85],
     RoundingRadius->5,
     StripOnInput->False],
    Deploy,
    DefaultBaseStyle->"Deploy"],
   If[28804050224869316589 === $SessionID, 
    Out[301], Message[
      MessageName[Syntax, "noinfoker"]]; Missing["NotAvailable"]; Null]], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.615258044118631*^9, 3.77767372505145*^9, 
  3.7823552220318255`*^9, 3.7823554003345547`*^9, 3.7823554646923914`*^9, 
  3.782355514761217*^9, 3.782355575152901*^9, 3.7823556403302736`*^9, 
  3.7823556988166285`*^9, 3.782355807346006*^9, 3.783952466310174*^9, 
  3.7839532579247217`*^9},ExpressionUUID->"3e8adfa8-6b52-42c6-9e10-\
d15550a899ff"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"E7", ",", 
   RowBox[{"{", 
    RowBox[{"SU8", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU3"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"F4", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"G2", ",", "Sp6"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "G2"}], "]"}], ",", "SU3", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "28000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.615287783986125*^9, 3.615287811418108*^9}, {
  3.615311804753172*^9, 3.615311808594244*^9}, {3.616237055029337*^9, 
  3.616237059634656*^9}, {3.616239089398303*^9, 3.6162390944929943`*^9}, {
  3.7776740507055683`*^9, 
  3.7776741494982977`*^9}},ExpressionUUID->"00f2d204-2b2f-4544-bcc1-\
a524f3f1ea2a"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"125.0\"\>",
      124.865899,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[124.865899, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616299554560072*^9, 
  3.777674286279377*^9},ExpressionUUID->"effcf1f5-172a-4a1b-92d2-\
662eae55e220"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "92", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox[
           SubscriptBox["\"E\"", "7"], TraditionalForm], 
          "\" Branching Rules\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["36", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["420",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["420", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["336",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["336", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["378", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["420",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["420", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1512", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1232",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1764",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3584",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["378", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["945",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["945", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2352",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3584",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1512", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["2520",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["2520", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8100",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["8100", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["36", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["420",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["420", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1176",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1176", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2100",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["2100", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8820",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["8820", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4224",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4752",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["4928",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4928",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["840", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["980",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["840",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["840", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox["\<\"F\"\>", "4"],
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["1053",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1274",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2652",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["52",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["273",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["273",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1274",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4096",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox["\<\"G\"\>", "2"],
                TraditionalForm], "\[Times]", 
               FormBox["\<\"Sp(6)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["14",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["350",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox[
                SubscriptBox["\<\"G\"\>", "2"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(3)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["125",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["80", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["260", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["91", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["162",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["162", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["216",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["343",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["125",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["143",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["143", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["154", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["216",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["343",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["260", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["405",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["405", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["440",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["440", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["80", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["81", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["91", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["260", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["343",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["405",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["405", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["440",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["440", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["55", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["80", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["143",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["143", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["162",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["162", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["270",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["270", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["343",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["405",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["405", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["595",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["595", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["143",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["143", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["190",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["190", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["405", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["405",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["405", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["440", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["442",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["442", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["595",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["595", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["648",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["648", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["910",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["910", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["136",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["136", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["143",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["143", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["270",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["270", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["405", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["440",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["440", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["442",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["442", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["595",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["595", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["648",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["648", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["836",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["836", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"12", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"12", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"13", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"12", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"13", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"12", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"10", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"10", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"16", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"14", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"20", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"17", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"14", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"22", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"25", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"14", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"13", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"25", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"18", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"21", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"21", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"10", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"16", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"16", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"13", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"17", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"13", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"10", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"15", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"10", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"24", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"22", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"10", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"33", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"24", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"21", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"34", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"35", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"18", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"19", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"34", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"26", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"30", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"10", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"14", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"24", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"29", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"14", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"20", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"21", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"14", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"20", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"13", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"14", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"12", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615288301833652*^9, 3.615793938621962*^9, 
  3.6162261123709106`*^9, 3.6162310137997503`*^9, 3.6162995562584267`*^9, 
  3.7776742877324805`*^9},ExpressionUUID->"5f7120df-c68f-419d-9083-\
62c0add57103"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"E7", ",", 
   RowBox[{"{", "SU2", "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "28000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7776741753197746`*^9, 3.7776741781192913`*^9}, {
  3.7788401798138833`*^9, 
  3.7788401799924116`*^9}},ExpressionUUID->"07735f2f-f8b5-44c8-8ba9-\
4110195f4cd3"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"53.3\"\>",
      53.3205473,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[53.3205473, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.777674314496875*^9, 
  3.7788402349012313`*^9},ExpressionUUID->"3cb8d079-0fd3-4431-8271-\
3c36acbf6fd7"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "29", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox[
           SubscriptBox["\"E\"", "7"], TraditionalForm], 
          "\" Branching Rules\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["18",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["34",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["38",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["40",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["42",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["58",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["62",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["69",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["67",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"39", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"39", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"38", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"40", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"38", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["68",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["72",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["74",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["76",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["78",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["82",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"39", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"45", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"47", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"51", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"50", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"50", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"50", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"45", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"44", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"41", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"37", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["62",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["68",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["72",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["76",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["12",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["22",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["38",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["37",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["43",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["41",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["46",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["53",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["49",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"40", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"51", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"56", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"60", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"66", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"67", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"66", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"68", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"64", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"60", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"58", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"52", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"46", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["58",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"44", "(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"55", "(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"68", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"75", "(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"80", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"87", "(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"87", "(", 
                FormBox[
                 StyleBox["18",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"86", "(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"85", "(", 
                FormBox[
                 StyleBox["22",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"80", "(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"74", "(", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"69", "(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"61", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"53", "(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"46", "(", 
                FormBox[
                 StyleBox["34",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"39", "(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["38",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["46",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["58",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.7776743146793528`*^9, 
  3.7788402350916967`*^9},ExpressionUUID->"7cea4297-b966-4e83-a29d-\
fbce2ad5cc46"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"E8", ",", 
   RowBox[{"{", 
    RowBox[{"SO16", ",", "SU9", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"E7", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"E6", ",", "SU3"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU5"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"G2", ",", "F4"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU3"}], "]"}], ",", "Sp4"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "30380"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.777674645974184*^9, 3.7776746727881207`*^9}, {
   3.7776748924187107`*^9, 3.7776748943804674`*^9}, {3.7839532297501364`*^9, 
   3.7839532312072616`*^9}, {3.7839546534733367`*^9, 3.783954654462694*^9}, 
   3.7839547268746195`*^9},ExpressionUUID->"6b938227-965d-4b0b-90ab-\
1eba69aa691e"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"41.7\"\>",
      41.7189274,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[41.7189274, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152774953049173`*^9, 3.777674847545083*^9, 
  3.777676036007248*^9, 3.7839545664683576`*^9, 3.7839547005218635`*^9, 
  3.7839547711950607`*^9},ExpressionUUID->"7fa76943-711c-4c7f-9bbb-\
192ee3bb2a2f"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "244", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox[
           SubscriptBox["\"E\"", "8"], TraditionalForm], 
          "\" Branching Rules\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["135",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1820",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1920",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1820",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5304",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["6435", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13312",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1920",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7020",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8008",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13312",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["84", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["240",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["240", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1050",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1050", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1215",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["84", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1050",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1050", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1215",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1944",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2520",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["2520", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5346",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["5346", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5760",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["84", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["240",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["240", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1050",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1050", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1215",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1540",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1540", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3402",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["3402", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5346",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["5346", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5760",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox["\<\"E\"\>", "7"],
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["133",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["133",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["912",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1539",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["133",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1463",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1539",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6480",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7371",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["133",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["133",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["912",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1463",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1539",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6480",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8645",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox["\<\"E\"\>", "6"],
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["351", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["351", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["351",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["351", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2430",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["27", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["351", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["351",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["351", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["351", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox["\<\"G\"\>", "2"],
                TraditionalForm], "\[Times]", 
               FormBox[
                SubscriptBox["\<\"F\"\>", "4"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["77",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["1053",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["52",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["26",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["324",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1053",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1274",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"12", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"8", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"9", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"6", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"12", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"11", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"7", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"Sp(4)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["5",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["14",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["30",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["154",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["165",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["260",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["390",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["429",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["14",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["204",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["285",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["390",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["455",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["595",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["810",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["935",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["1190",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1326",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1330",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["30",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["390",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["429",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["455",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["595",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["625",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["715",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["810",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["836",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["935",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1190",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["1326",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615277497405105*^9, 3.77767484834694*^9, 
  3.7776760373017874`*^9, 3.7839545668084803`*^9, 3.783954700714376*^9, 
  3.7839547715371466`*^9},ExpressionUUID->"7b49d791-2d78-4ea0-9d22-\
1e51d43535f9"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"E8", ",", 
   RowBox[{"{", "SU2", "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "30380"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7776747413442106`*^9, 3.777674745963422*^9}, 
   3.777676109436785*^9, 3.778839535495176*^9, {3.7839546059056177`*^9, 
   3.7839546071999536`*^9}},ExpressionUUID->"c1847ea3-fafb-4acd-af82-\
292514fda463"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"18.3\"\>",
      18.3409984,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[18.3409984, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7776760921200805`*^9, 3.7776762154115853`*^9, 
  3.778840062018174*^9, 
  3.7839546262230268`*^9},ExpressionUUID->"c30c97b3-db4d-4944-a095-\
d511c45bda9c"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "242", "\": \""}, 
          "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
          FormBox[
           SubscriptBox["\"E\"", "8"], TraditionalForm], 
          "\" Branching Rules\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["65",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["69",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["73",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"28", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["83",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["89",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["93",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"37", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"40", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"40", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"38", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"36", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"30", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["85",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["87",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["91",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["11",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["17",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["29",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["51",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["55",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["57",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["61",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"31", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"40", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"33", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"47", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"39", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"50", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"43", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"51", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"50", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"40", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"46", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"37", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"41", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"32", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["71",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"29", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"39", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"38", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"45", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"55", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"50", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"57", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"52", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"57", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"49", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"54", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"46", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"48", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"41", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"42", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"34", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"35", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"27", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"26", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["69",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["73",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["75",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["3",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["23",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["39",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["47",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["59",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["19",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["67",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["71",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["79",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["81",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["85",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["93",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"14", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"8", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"6", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["95",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["97",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["99",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["101",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["103",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["109",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["113",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["117",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"21", "(", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"18", "(", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"23", "(", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"25", "(", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"24", "(", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"22", "(", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"20", "(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"17", "(", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"19", "(", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"16", "(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"12", "(", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"15", "(", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"11", "(", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"13", "(", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"10", "(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"9", "(", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"7", "(", 
                FormBox[
                 StyleBox["83",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["85",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["87",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["89",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"5", "(", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["93",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", "(", 
                FormBox[
                 StyleBox["95",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["97",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["101",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               RowBox[{"2", "(", 
                FormBox[
                 StyleBox["103",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["107",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["111",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["115",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.777676092193881*^9, 3.777676215550212*^9, 
  3.7788400623433185`*^9, 
  3.783954627267268*^9},ExpressionUUID->"51b547be-6196-482a-95f2-\
c44bdfa83dd3"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"G2", ",", 
   RowBox[{"{", 
    RowBox[{"SU3", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2"}], "]"}], ",", "SU2"}], "}"}], ",", " ", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "13"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "14756"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774615516908679*^9, 3.7746155181982913`*^9}, {
  3.7746156217945642`*^9, 
  3.7746156230970554`*^9}},ExpressionUUID->"f7fbd80d-a383-4491-9e36-\
f84a3373db45"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"149.0\"\>",
      148.6478551,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[148.6478551, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.776706211247904*^9, 
  3.777677404962535*^9},ExpressionUUID->"78f26a53-c201-42fd-888e-\
7249f1dd5a8e"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "109", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox[
            SubscriptBox["\"G\"", "2"], TraditionalForm], 
           "\" Branching Rules\""}, "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox[
                SubscriptBox["\<\"G\"\>", "2"],
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"27", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"64", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"182", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"189", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"273", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"286", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"714", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"748", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"896", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"924", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1254", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1547", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2261", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2926", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3289", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["99", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3542", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4914", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["80", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["99", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["99", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["120",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5103", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6630", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["420", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7293", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7371", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["78", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["99", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["120",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["143", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["195", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["234",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["234", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7722", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["80", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["99",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["99", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["120",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["132", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8372", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["80", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["99", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["132", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9177", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9660", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["273",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["273", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["343",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["420", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10206", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["273",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["273", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10556", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["78", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["99", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["120",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["143", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["168", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["195", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["231", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["234",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["234", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11571", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["99", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["234",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["234", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["405", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["420", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["504",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["504", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["612",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["612", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["729",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11648", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["80", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["99",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["99", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["120",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["120", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["132", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["143", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["162",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["162", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["192",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["192", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["195", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["234",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["234", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["80", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["99", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["132", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["192",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["192", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["234",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["234", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13090", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["63", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["80",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["80", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["81",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["81", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["99",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["99", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["105",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["120",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["132",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["132", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["154",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["154", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["162",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["162", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["165",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["165", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["192",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["192", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["195", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["210",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["234",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["234", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14756", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["28", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["42",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["42", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["48", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["55", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["63", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["66", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["78", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["80", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["81", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["90",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["90", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["91",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["91", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["99", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["120",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["120", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["125",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["132",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["132", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["143",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["143", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["154",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["154", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["162",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["162", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["165", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["168", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["192",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["192", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["195",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["195", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["195",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["195", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["231", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["234",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["234", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["260",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["260", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["270",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["270", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["273",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["273", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["330", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["343",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["375",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["375", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["405", "_"],
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox[
                  OverscriptBox["420", "_"],
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox[
                SubscriptBox["\<\"G\"\>", "2"],
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"27", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"64", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"182", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"189", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"273", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"286", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"714", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"748", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"896", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"924", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1254", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1547", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2261", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2926", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3289", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3542", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4914", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5103", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6630", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7293", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7371", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7722", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8372", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9177", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9660", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["18",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10206", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10556", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["18",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["19",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11571", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["18",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["19",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11648", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["18",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["18",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["19",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13090", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["18",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["18",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14756", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["18",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["12",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["13",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["16",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["17",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["18",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["19",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["20",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["4",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["12",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["18",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["22",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["26",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox[
                SubscriptBox["\<\"G\"\>", "2"],
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"27", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"64", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"77", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"182", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"189", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"273", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"286", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"378", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"714", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"748", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"896", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"924", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1254", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1547", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1728", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1729", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2079", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2261", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2926", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3003", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3289", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"3542", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4914", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4928", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5005", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"5103", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"6630", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["59",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["61",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["71",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7293", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7371", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["59",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"7722", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8372", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9177", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"9660", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["59",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["61",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10206", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["59",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10556", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["59",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["61",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["69",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11571", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["59",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["61",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["67",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["69",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["71",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["73",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["81",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"11648", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["59",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"13090", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"27", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"27", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["59",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"14756", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"22", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["59",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["61",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["63",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["65",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["67",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["69",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["71",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["73",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["79",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.776706212379874*^9, 
  3.777677405986765*^9},ExpressionUUID->"87a974b3-becd-49b8-9e72-\
5c46d8916650"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"F4", ",", 
   RowBox[{"{", 
    RowBox[{"SO9", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "SU3"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "Sp6"}], "]"}], ",", "SU2", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "G2"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "19448"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "20"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.774568016082088*^9, 3.774568050754381*^9}, {
  3.7767066443439093`*^9, 3.776706645863805*^9}, {3.782356350472426*^9, 
  3.782356351732027*^9}, {3.7823566662189827`*^9, 3.7823566680730233`*^9}, {
  3.783955195791484*^9, 3.7839551971414137`*^9}, {3.783955512405748*^9, 
  3.7839555136204996`*^9}},ExpressionUUID->"6725efba-b661-4e4a-be52-\
af8f8f3d3d97"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"51.1\"\>",
      51.1290922,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[51.1290922, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.7767063344981155`*^9, 3.776706710527774*^9, 
  3.7823564068103275`*^9, 3.782356698193878*^9, 3.783955341671425*^9, 
  3.7839555656499968`*^9},ExpressionUUID->"afdb230c-d8b9-46a1-832d-\
69216239a33f"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  FormBox[
   TagBox[GridBox[{
      {
       TemplateBox[{StyleBox[
          
          TemplateBox[{"\"Table\[MediumSpace]\"", "248", "\": \""}, 
           "RowDefault"], Bold, StripOnInput -> False],TemplateBox[{
           FormBox[
            SubscriptBox["\"F\"", "4"], TraditionalForm], 
           "\" Branching Rules\""}, "RowDefault"]},
        "RowDefault"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox[
                SubscriptBox["\<\"F\"\>", "4"],
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SO(9)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"273", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"324", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1053", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1053", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1274", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2652", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["128",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8424", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2560",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10829", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["910",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2772",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12376", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2772",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4004",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4928",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["156",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["2772",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3900",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4608",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"17901", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["672",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2574",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2772",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["4928",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"19278", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["432",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2457",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2772",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5040",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"19448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["44",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["231",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["432",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["576",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["768",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["924",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1650",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["1980",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["2560",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["3900",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["5040",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox[
                SubscriptBox["\<\"F\"\>", "4"],
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(3)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"SU(3)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"273", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"324", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1053", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1053", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1274", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2652", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8424", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["42",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["42", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10829", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["42",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["42", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12376", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["42",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["42", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["90", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["125",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"17901", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["35", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["42",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["42", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["42",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["42", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"19278", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"8", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["35", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["35", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["42",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["42", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["42",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["42", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"19448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["10",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["10", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["21", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["28",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["28", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["15",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["6", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["6",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["35", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["24",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["15",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["24", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["15", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["24", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["35",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["35", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["8",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["42",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["42", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox[
                    OverscriptBox["3", "_"],
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["48", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["42",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["42", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["60",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["60", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox[
                SubscriptBox["\<\"F\"\>", "4"],
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox["\<\"Sp(6)\"\>",
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"273", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"324", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1053", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1053", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1274", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2652", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["385",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8424", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["70",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["525",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10829", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   SuperscriptBox[
                    StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["90",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["216",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12376", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["330",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["462",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1078",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1100",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["90",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["385",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1274",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1344",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"17901", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["70",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["126",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["252",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["385",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["525",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["525",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["924",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"19278", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["56",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["70",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["70",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["126",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["189",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["189",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["350",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["378",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["385",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["525",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["525",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["594",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"19448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["14",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["21",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["56",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["90",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["90",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["126",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["126",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["189",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["2",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["216",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["216",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["350",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["385",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["448",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["512",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["525",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["616",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["924",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1078",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1386",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox[
                SubscriptBox["\<\"F\"\>", "4"],
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"273", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["17",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["25",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["31",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"324", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["19",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["23",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["29",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1053", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["33",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1053", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["41",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1274", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["37",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["39",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2652", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["49",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["43",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["47",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8424", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["51",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10829", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                RowBox[{"11", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"31", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"35", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["53",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12376", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"24", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"31", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["59",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["67",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"37", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"43", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"37", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"44", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"38", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"42", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"35", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"38", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"30", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"19", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["65",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"17901", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"33", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"36", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"42", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"44", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"48", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"47", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"49", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"46", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"46", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"42", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"41", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"36", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"33", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"28", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"18", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"12", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"7", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"19278", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"16", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"33", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"36", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"46", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"46", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"54", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"53", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"58", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"53", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"55", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"50", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"50", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"43", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"41", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"34", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"32", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"23", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"17", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"15", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"10", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["57",
                  FontWeight->Bold],
                 TraditionalForm], "+", 
                FormBox[
                 StyleBox["59",
                  FontWeight->Bold],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"19448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"9", "(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"25", "(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"26", "(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"40", "(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"38", "(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"51", "(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"47", "(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"56", "(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"51", "(", 
                 FormBox[
                  StyleBox["19",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"57", "(", 
                 FormBox[
                  StyleBox["21",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"49", "(", 
                 FormBox[
                  StyleBox["23",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"54", "(", 
                 FormBox[
                  StyleBox["25",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"45", "(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"46", "(", 
                 FormBox[
                  StyleBox["29",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"38", "(", 
                 FormBox[
                  StyleBox["31",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"38", "(", 
                 FormBox[
                  StyleBox["33",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"29", "(", 
                 FormBox[
                  StyleBox["37",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"21", "(", 
                 FormBox[
                  StyleBox["39",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"20", "(", 
                 FormBox[
                  StyleBox["41",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"14", "(", 
                 FormBox[
                  StyleBox["43",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"13", "(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["47",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"8", "(", 
                 FormBox[
                  StyleBox["49",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["51",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", "(", 
                 FormBox[
                  StyleBox["53",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["55",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", "(", 
                 FormBox[
                  StyleBox["57",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                FormBox[
                 StyleBox["61",
                  FontWeight->Bold],
                 TraditionalForm]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]},
      {
       TagBox[GridBox[{
          {
           TagBox[GridBox[{
              {
               FormBox[
                SubscriptBox["\<\"F\"\>", "4"],
                TraditionalForm], 
               FormBox["\<\"\[RightArrow]\"\>",
                TraditionalForm], 
               RowBox[{
                FormBox["\<\"SU(2)\"\>",
                 TraditionalForm], "\[Times]", 
                FormBox[
                 SubscriptBox["\<\"G\"\>", "2"],
                 TraditionalForm]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"26", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"52", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"273", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"324", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1053", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1053", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"1274", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"2652", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"4096", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"8424", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"10829", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"12376", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["273",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"16302", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["17",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"17901", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["286",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"19278", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["14",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"6", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]},
              {
               FormBox[
                TagBox[
                 StyleBox[
                  RowBox[{"19448", "\[InvisibleSpace]", 
                   SuperscriptBox["\[AlignmentMarker]", ""]}],
                  StripOnInput->False,
                  FontWeight->Bold],
                 DisplayForm],
                TraditionalForm], 
               FormBox["\<\"=\"\>",
                TraditionalForm], 
               RowBox[{
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"5", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["11",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["14",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["1",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"7", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["27",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["13",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"4", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["9",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["64",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["11",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"3", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["3",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["5",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"2", 
                 RowBox[{"(", 
                  FormBox[
                   StyleBox["7",
                    FontWeight->Bold],
                   TraditionalForm], ",", 
                  FormBox[
                   StyleBox["77",
                    FontWeight->Bold],
                   TraditionalForm], ")"}]}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["77",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["9",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["77",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["182",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}], "+", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["189",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}]}
             },
             AutoDelete->False,
             
             GridBoxAlignment->{
              "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
               "Rows" -> {Center, {Center}}},
             GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                 RGBColor[0.8, 0.8, 0.8], {
                  GrayLevel[1], 
                  RGBColor[0.9, 0.9, 0.9]}}},
             GridBoxDividers->{"Columns" -> {
                 GrayLevel[1], {False}, 
                 GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             
             GridBoxSpacings->{
              "Columns" -> {1, 0.25, 0.5, 2}, 
               "Rows" -> {1, 1, {Automatic}, 0.5}}],
            "Grid"]}
         },
         AutoDelete->False,
         GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
         GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
         GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
        "Grid"]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
     GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
     GridBoxSpacings->{
      "Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
    "Column"],
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.776706334525071*^9, 3.7767067112608433`*^9, 
  3.7823564071603603`*^9, 3.7823566984950743`*^9, 3.7839553420853167`*^9, 
  3.7839555660060697`*^9},ExpressionUUID->"fa2cec5b-8e3b-4e23-8e7c-\
2d174c617152"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.777564585056141*^9, 
  3.777564591784156*^9}},ExpressionUUID->"eb6ea702-8360-4677-bf94-\
f9adb5798f1b"]
},
CellGrouping->Manual,
WindowSize->{869, 768},
WindowMargins->{{269, Automatic}, {Automatic, 37}},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> True}},
CommonDefaultFormatTypes->{"Output"->TraditionalForm},
FrontEndVersion->"11.1 for Microsoft Windows (64-bit) (April 18, 2017)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 97, 0, 83, "Title", "ExpressionUUID" -> \
"0b4c7478-88ab-43fa-8576-f7e31bd8b9e7"],
Cell[679, 24, 273, 8, 30, "Subsubtitle", "ExpressionUUID" -> \
"d7b83ecd-1c48-45de-a6a5-8ed6a04c7a0f"],
Cell[CellGroupData[{
Cell[977, 36, 88, 0, 71, "Section", "ExpressionUUID" -> \
"dcc38c6f-cfa2-42c9-aea8-6d499dbb4386"],
Cell[CellGroupData[{
Cell[1090, 40, 143, 3, 29, "Input", "ExpressionUUID" -> \
"381f412f-111f-410a-9c54-2f74acb1dfa4",
 InitializationCell->True],
Cell[1236, 45, 220, 4, 70, "Print", "ExpressionUUID" -> \
"f6bd3fbb-c159-41f4-af76-d0506601f42b"],
Cell[1459, 51, 236, 4, 70, "Print", "ExpressionUUID" -> \
"384d90db-6c8b-4c42-b504-2f871ea0a691"]
}, Open  ]],
Cell[1710, 58, 392, 11, 68, "Input", "ExpressionUUID" -> \
"90bcb6f0-ae5d-4ed1-9002-bfa594defbe0",
 InitializationCell->True],
Cell[2105, 71, 280, 6, 31, "Input", "ExpressionUUID" -> \
"b8b56ffc-5773-4fec-bf57-0f3b55946d09"]
}, Closed]],
Cell[CellGroupData[{
Cell[2422, 82, 79, 0, 41, "Section", "ExpressionUUID" -> \
"b3b0012b-3662-4eca-8162-00d136cab5da"],
Cell[CellGroupData[{
Cell[2526, 86, 447, 9, 29, "Input", "ExpressionUUID" -> \
"2b166e86-bbec-49a3-b1fc-6e280ac57893"],
Cell[2976, 97, 580, 16, 23, "Print", "ExpressionUUID" -> \
"2ad7ff5b-4bb5-4cc3-87ae-1e4fb253dd6a"],
Cell[3559, 115, 48494, 1290, 740, "Output", "ExpressionUUID" -> \
"ed03fde6-02a1-4ca6-8a37-6fe46f57cc4c"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52090, 1410, 625, 13, 29, "Input", "ExpressionUUID" -> \
"894fe9eb-3835-4c8c-a404-97c7f89d598d"],
Cell[52718, 1425, 715, 17, 23, "Print", "ExpressionUUID" -> \
"042b0e6d-10c3-4a02-b4a6-7a4e5e9fb928"],
Cell[53436, 1444, 167111, 4599, 1496, "Output", "ExpressionUUID" -> \
"eab70b92-56f8-41e8-9fbd-0eae2a39b1ed"]
}, Open  ]],
Cell[CellGroupData[{
Cell[220584, 6048, 776, 17, 48, "Input", "ExpressionUUID" -> \
"762fc1bc-6111-45a0-a466-637c2ab9d288"],
Cell[221363, 6067, 663, 17, 23, "Print", "ExpressionUUID" -> \
"2afbc7c4-5a16-483c-9493-0a739de7859d"],
Cell[222029, 6086, 146364, 4123, 1576, "Output", "ExpressionUUID" -> \
"19b64b38-1fc6-48c8-b4fc-9a7c638e37b5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[368430, 10214, 692, 15, 48, "Input", "ExpressionUUID" -> \
"68fe7032-01cf-4af0-a2ef-3e3d75947855"],
Cell[369125, 10231, 612, 16, 23, "Print", "ExpressionUUID" -> \
"0e36d88b-941b-4e29-9bc2-20866c4658e4"],
Cell[369740, 10249, 141124, 3915, 1490, "Output", "ExpressionUUID" -> \
"b2d48cbd-f2a9-49da-a7b0-df088a64261d"]
}, Open  ]],
Cell[CellGroupData[{
Cell[510901, 14169, 907, 20, 66, "Input", "ExpressionUUID" -> \
"e7faadef-8faf-4e4f-9347-a6957d90480d"],
Cell[511811, 14191, 683, 17, 23, "Print", "ExpressionUUID" -> \
"960f7e0a-c44a-4b9e-92b6-944f6743e601"],
Cell[512497, 14210, 300085, 8494, 3296, "Output", "ExpressionUUID" -> \
"5a73be9f-3003-4902-84b0-175b508613b3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[812619, 22709, 741, 16, 48, "Input", "ExpressionUUID" -> \
"af66838a-c06d-474e-a19f-662c729d8df1"],
Cell[813363, 22727, 584, 15, 23, "Print", "ExpressionUUID" -> \
"ed45a218-d538-4db1-806f-c0b2074a6079"],
Cell[813950, 22744, 279668, 7744, 2604, "Output", "ExpressionUUID" -> \
"ce9bc265-7da2-4eb3-bd3a-f866cd82a370"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1093655, 30493, 993, 22, 66, "Input", "ExpressionUUID" -> \
"ed825859-5a01-44b2-a32e-0d57e5d2153a"],
Cell[1094651, 30517, 583, 15, 23, "Print", "ExpressionUUID" -> \
"3a0ea1f7-6503-4a21-9926-366b37b29423"],
Cell[1095237, 30534, 310438, 8718, 3192, "Output", "ExpressionUUID" -> \
"b488b4c6-c8e4-4649-8631-e67a60058eb2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1405712, 39257, 995, 21, 66, "Input", "ExpressionUUID" -> \
"98007754-71ba-409d-b396-dd3cb47d8f36"],
Cell[1406710, 39280, 604, 15, 23, "Print", "ExpressionUUID" -> \
"18bbaf07-7eb4-48b1-892c-97257497b084"],
Cell[1407317, 39297, 262959, 7377, 2480, "Output", "ExpressionUUID" -> \
"30e2d29b-cf71-4c44-a941-090e2200f252"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1670313, 46679, 1040, 23, 66, "Input", "ExpressionUUID" -> \
"08b23165-591c-4e27-8ff3-9d2ba1de2783"],
Cell[1671356, 46704, 583, 15, 23, "Print", "ExpressionUUID" -> \
"1e279120-92b0-4f04-bee6-e481867d5046"],
Cell[1671942, 46721, 343379, 9772, 4140, "Output", "ExpressionUUID" -> \
"40973258-04b3-4ee8-93df-b9f1bd8a133f"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2015358, 56498, 1065, 23, 66, "Input", "ExpressionUUID" -> \
"f0d228c9-a713-4ceb-92a7-e707d084d931"],
Cell[2016426, 56523, 660, 16, 23, "Print", "ExpressionUUID" -> \
"c8b25173-035e-4b17-ae76-b55bc85d752b"],
Cell[2017089, 56541, 148275, 4130, 1636, "Output", "ExpressionUUID" -> \
"a566e394-03cf-4174-b7e8-5058b6a43466"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2165401, 60676, 1101, 25, 66, "Input", "ExpressionUUID" -> \
"2fc0d887-8500-4152-824c-6da41c8c9258"],
Cell[2166505, 60703, 558, 15, 23, "Print", "ExpressionUUID" -> \
"af05d8c5-3f23-47a5-b048-8e7856e3b125"],
Cell[2167066, 60720, 216715, 6126, 2600, "Output", "ExpressionUUID" -> \
"7767be2d-9642-4664-8383-cd4fce1b26a4"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2383830, 66852, 994, 23, 66, "Input", "ExpressionUUID" -> \
"14ed25d7-27d9-4a62-89e2-157d6448ed68"],
Cell[2384827, 66877, 391, 8, 21, "Message", "ExpressionUUID" -> \
"f5788445-af66-435a-b65a-e63eafcb44fb"],
Cell[2385221, 66887, 516, 14, 23, "Print", "ExpressionUUID" -> \
"15cd0dba-9b53-4dc0-a847-8f0311c3f761"],
Cell[2385740, 66903, 149010, 4150, 1630, "Output", "ExpressionUUID" -> \
"83bb7774-47f5-4e84-b8b2-2bd37052b46c"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2534787, 71058, 1242, 27, 84, "Input", "ExpressionUUID" -> \
"6c8308e3-6a31-42a0-aad7-acb9243c19e7"],
Cell[2536032, 71087, 515, 14, 23, "Print", "ExpressionUUID" -> \
"af581579-75ea-4af1-bd6e-67b671cb041b"],
Cell[2536550, 71103, 108774, 3054, 1520, "Output", "ExpressionUUID" -> \
"c8003458-0871-4707-b625-ba5b4cf25a33"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2645361, 74162, 1400, 29, 84, "Input", "ExpressionUUID" -> \
"93caca86-03d2-465e-81ba-fa912fde7f2e"],
Cell[2646764, 74193, 588, 15, 23, "Print", "ExpressionUUID" -> \
"9048ccc1-ca73-4aac-bd59-f47a4858c16a"],
Cell[2647355, 74210, 163196, 4586, 2120, "Output", "ExpressionUUID" -> \
"7e2626bc-22e4-44a6-9084-2cd664cec4a6"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2810588, 78801, 508, 10, 29, "Input", "ExpressionUUID" -> \
"2ff77ce0-33f0-423c-b37a-b31a9130451d"],
Cell[2811099, 78813, 585, 15, 23, "Print", "ExpressionUUID" -> \
"41e32968-409b-477e-a66a-10b205548309"],
Cell[2811687, 78830, 29722, 868, 420, "Output", "ExpressionUUID" -> \
"73ab3a39-292a-4b94-abf3-388a526d9f4b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2841446, 79703, 1413, 31, 102, "Input", "ExpressionUUID" -> \
"1dd7acfa-8b79-4ecd-9165-186a77c45b3d"],
Cell[2842862, 79736, 372, 9, 21, "Message", "ExpressionUUID" -> \
"da9ef50a-aa24-45d5-9a68-7472eb5ae7b5"],
Cell[2843237, 79747, 520, 14, 23, "Print", "ExpressionUUID" -> \
"d74c7e75-f368-48dc-b19b-8e80c17ffcbd"],
Cell[2843760, 79763, 109529, 3085, 1744, "Output", "ExpressionUUID" -> \
"4d07fa0d-17c4-4a28-953a-a589a3f609da"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2953326, 82853, 79, 0, 71, "Section", "ExpressionUUID" -> \
"d894e233-7c41-4415-bb6d-75557baebd5a"],
Cell[CellGroupData[{
Cell[2953430, 82857, 653, 14, 48, "Input", "ExpressionUUID" -> \
"46655597-d117-4ba9-9c85-83eaa626a43b"],
Cell[2954086, 82873, 561, 15, 23, "Print", "ExpressionUUID" -> \
"ba93dcec-303a-4000-9b5a-73d475bdd581"]
}, Open  ]],
Cell[2954662, 82891, 1064884, 29665, 6524, "Output", "ExpressionUUID" -> \
"8399acee-938a-42f7-a364-a4d2115127c8"],
Cell[CellGroupData[{
Cell[4019571, 112560, 756, 16, 48, "Input", "ExpressionUUID" -> \
"6c2b9ec3-0da1-4c4a-af2d-f46729fc5e52"],
Cell[4020330, 112578, 561, 15, 23, "Print", "ExpressionUUID" -> \
"2f0eb717-56c9-4cd8-9277-2f3d25e021e5"]
}, Open  ]],
Cell[4020906, 112596, 2563009, 72061, 15018, "Output", "ExpressionUUID" -> \
"2354a838-0dca-4106-aa88-67e6791d6c43"],
Cell[CellGroupData[{
Cell[6583940, 184661, 811, 18, 48, "Input", "ExpressionUUID" -> \
"ece41d6b-c031-46bc-a96f-9fdcfb77a421"],
Cell[6584754, 184681, 583, 15, 23, "Print", "ExpressionUUID" -> \
"1ceb3d66-1771-4371-84c5-0ea30716015a"]
}, Open  ]],
Cell[6585352, 184699, 1102942, 31019, 7016, "Output", "ExpressionUUID" -> \
"c0c52fb4-9f14-40cf-b1f3-9d90b4f15ed3"],
Cell[CellGroupData[{
Cell[7688319, 215722, 1016, 22, 66, "Input", "ExpressionUUID" -> \
"404d29c9-b272-4289-a9e7-5b6f211e186a"],
Cell[7689338, 215746, 609, 15, 23, "Print", "ExpressionUUID" -> \
"7ca12e1d-8e47-4690-8f4d-7ad923737649"]
}, Open  ]],
Cell[7689962, 215764, 712996, 19768, 5472, "Output", "ExpressionUUID" -> \
"239fd7b1-41cc-42e4-b95d-2c5d29d81ac1"],
Cell[CellGroupData[{
Cell[8402983, 235536, 932, 20, 48, "Input", "ExpressionUUID" -> \
"d7f23682-9c10-4061-bc1c-45577f44a779"],
Cell[8403918, 235558, 583, 15, 23, "Print", "ExpressionUUID" -> \
"e0227d2d-8839-4e31-9e01-e925b8f270e9"],
Cell[8404504, 235575, 387581, 11225, 3084, "Output", "ExpressionUUID" -> \
"d9355c55-0b59-4979-afe8-bae24fe2067e"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8792122, 246805, 1129, 26, 84, "Input", "ExpressionUUID" -> \
"d21cfb39-b7d1-42cc-a5e5-9e45789a20af"],
Cell[8793254, 246833, 537, 14, 23, "Print", "ExpressionUUID" -> \
"f446333c-285b-4c2b-9509-5920a39278d0"]
}, Open  ]],
Cell[8793806, 246850, 1307144, 36632, 8398, "Output", "ExpressionUUID" -> \
"ede9bf87-0452-49b6-8cff-394fb2eabe88"],
Cell[CellGroupData[{
Cell[10100975, 283486, 825, 19, 66, "Input", "ExpressionUUID" -> \
"bfc6706b-3d3c-430c-a296-4a4d64edd8c1"],
Cell[10101803, 283507, 510, 14, 23, "Print", "ExpressionUUID" -> \
"9b001e4a-e0d2-4a95-a41a-3b363979dc73"]
}, Open  ]],
Cell[10102328, 283524, 610222, 17183, 4452, "Output", "ExpressionUUID" -> \
"f291319c-9c02-462b-9206-64bb8ebe5700"],
Cell[CellGroupData[{
Cell[10712575, 300711, 1018, 24, 66, "Input", "ExpressionUUID" -> \
"bef074f0-3ec6-4ac5-8dc5-a76ec69bb67d"],
Cell[10713596, 300737, 514, 14, 23, "Print", "ExpressionUUID" -> \
"053d11e9-82ae-4ab6-9932-2271e38d993d"],
Cell[10714113, 300753, 463104, 13379, 4484, "Output", "ExpressionUUID" -> \
"73f63d58-5c2d-41ec-9d3a-4b608d97289f"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11177254, 314137, 1136, 25, 66, "Input", "ExpressionUUID" -> \
"faa7a6f0-fec0-469c-b43e-503332e3882d"],
Cell[11178393, 314164, 512, 14, 23, "Print", "ExpressionUUID" -> \
"de4e8b2a-a32f-41f1-a4ba-852ecb661ebb"]
}, Open  ]],
Cell[11178920, 314181, 493371, 13955, 4110, "Output", "ExpressionUUID" -> \
"cfc23f7a-a0cc-4c99-be84-0ab2a279321c"],
Cell[CellGroupData[{
Cell[11672316, 328140, 1302, 30, 84, "Input", "ExpressionUUID" -> \
"75b1ab77-69ea-404a-a41a-3eceec034529"],
Cell[11673621, 328172, 512, 14, 23, "Print", "ExpressionUUID" -> \
"33672dc2-423d-4653-ab3a-385f2fa0be72"],
Cell[11674136, 328188, 319281, 9254, 3270, "Output", "ExpressionUUID" -> \
"2a6e7bf6-56dd-4969-8eb2-291d836fbf59"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11993454, 337447, 996, 23, 66, "Input", "ExpressionUUID" -> \
"82da776f-f6c6-48b3-84f7-a26f4c25ea1e"],
Cell[11994453, 337472, 488, 14, 23, "Print", "ExpressionUUID" -> \
"6c37db35-39dc-4443-9573-0e586f85766b"],
Cell[11994944, 337488, 227966, 6637, 2332, "Output", "ExpressionUUID" -> \
"dd3cbd60-0df7-41b4-b2ac-1db6723a1116"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12222947, 344130, 1236, 29, 84, "Input", "ExpressionUUID" -> \
"a294a84d-88df-47d6-9c48-67b7e63813d5"],
Cell[12224186, 344161, 512, 14, 23, "Print", "ExpressionUUID" -> \
"d33b8531-fe66-4ef0-a6eb-4fd4aba6749e"],
Cell[12224701, 344177, 210783, 6092, 2372, "Output", "ExpressionUUID" -> \
"3f9786c8-0138-49bd-9b05-08b488d722af"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12435521, 350274, 1184, 26, 84, "Input", "ExpressionUUID" -> \
"10705a46-b241-4669-b8b9-c8fbd6516675"],
Cell[12436708, 350302, 512, 14, 23, "Print", "ExpressionUUID" -> \
"2caeabee-6b62-43f9-87e3-a8510743901e"],
Cell[12437223, 350318, 194321, 5657, 2176, "Output", "ExpressionUUID" -> \
"6204cdd1-73a4-4157-8dd2-241363250db2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12631581, 355980, 1596, 36, 102, "Input", "ExpressionUUID" -> \
"923d140b-321d-4629-b9ef-532d927551d8"],
Cell[12633180, 356018, 563, 15, 23, "Print", "ExpressionUUID" -> \
"70ccc799-581d-414b-b693-abc81a0ba103"],
Cell[12633746, 356035, 289394, 8406, 3070, "Output", "ExpressionUUID" -> \
"e115786b-7f34-4d3c-80f9-4cacc7d7cb25"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12923177, 364446, 1461, 32, 102, "Input", "ExpressionUUID" -> \
"c25fd1a7-18de-4caa-836d-778e29c54c5e"],
Cell[12924641, 364480, 515, 14, 23, "Print", "ExpressionUUID" -> \
"86c5f5eb-acc9-4ea6-ad48-bfb78df3aff3"],
Cell[12925159, 364496, 296270, 8639, 3138, "Output", "ExpressionUUID" -> \
"3d8b5787-f813-4e27-8288-3175dfc2342c"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13221466, 373140, 1429, 32, 102, "Input", "ExpressionUUID" -> \
"67edb12f-825c-409d-84f2-8e80013cbbe5"],
Cell[13222898, 373174, 534, 14, 23, "Print", "ExpressionUUID" -> \
"cac4f490-4641-4a72-b696-63e3ca7ce887"],
Cell[13223435, 373190, 201013, 5818, 2266, "Output", "ExpressionUUID" -> \
"7cb5bdd9-0de5-418f-ae67-87509e81fad8"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13424485, 379013, 1413, 31, 84, "Input", "ExpressionUUID" -> \
"639363d3-e70b-4dde-9380-c49369f73a55"],
Cell[13425901, 379046, 534, 14, 23, "Print", "ExpressionUUID" -> \
"8f9ef471-0351-4e2b-a3c8-4f4313a472de"],
Cell[13426438, 379062, 175035, 5094, 2122, "Output", "ExpressionUUID" -> \
"193e2026-9c6b-4795-8a4c-cef002a7f93c"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13601510, 384161, 1653, 38, 120, "Input", "ExpressionUUID" -> \
"f7fb7f6b-0166-49ea-b7be-b47e682afdbe"],
Cell[13603166, 384201, 491, 14, 23, "Print", "ExpressionUUID" -> \
"cccfc549-e3b3-47a8-8c0c-c119b2935c04"],
Cell[13603660, 384217, 228152, 6617, 2752, "Output", "ExpressionUUID" -> \
"cc2d6505-4e1c-47a2-a29d-697ead76ab6b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13831849, 390839, 1531, 34, 102, "Input", "ExpressionUUID" -> \
"a7df1aa9-d7b0-415c-8e96-b2550879da39"],
Cell[13833383, 390875, 513, 14, 23, "Print", "ExpressionUUID" -> \
"10a7263f-f0df-4693-b154-f0cb9c371488"],
Cell[13833899, 390891, 277261, 8072, 2904, "Output", "ExpressionUUID" -> \
"2fa68fe2-9fb6-46a0-b8de-926f20ba9458"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14111197, 398968, 1512, 35, 102, "Input", "ExpressionUUID" -> \
"b3e02d00-2f37-4933-8327-2141014b6a14"],
Cell[14112712, 399005, 372, 9, 21, "Message", "ExpressionUUID" -> \
"3e920223-3cd1-44a1-b482-d865fd1e53ea"],
Cell[14113087, 399016, 518, 14, 23, "Print", "ExpressionUUID" -> \
"351a2abb-5b7c-455f-8227-409f17ce6817"]
}, Open  ]]
}, Open  ]],
Cell[14113632, 399034, 563122, 15833, 5018, "Output", "ExpressionUUID" -> \
"ab55466c-c65b-407f-99d6-9ac89bd7af71"],
Cell[CellGroupData[{
Cell[14676779, 414871, 1629, 38, 120, "Input", "ExpressionUUID" -> \
"d3c0b523-70bf-4900-8517-6546dd669391"],
Cell[14678411, 414911, 372, 9, 21, "Message", "ExpressionUUID" -> \
"b94635a0-9912-49ed-8903-8475b29ca93f"],
Cell[14678786, 414922, 494, 14, 23, "Print", "ExpressionUUID" -> \
"f00ecfe8-6873-400c-aedf-f606b925a756"],
Cell[14679283, 414938, 414860, 12071, 4096, "Output", "ExpressionUUID" -> \
"32f15d98-ca71-45c0-9c3a-cf6fa357e175"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15094180, 427014, 1853, 43, 120, "Input", "ExpressionUUID" -> \
"f249b961-10d6-489d-abe4-5349249fda1e"],
Cell[15096036, 427059, 374, 9, 21, "Message", "ExpressionUUID" -> \
"c757c345-a00c-4479-9837-41ad01697262"],
Cell[15096413, 427070, 496, 14, 23, "Print", "ExpressionUUID" -> \
"8a1e674c-a7c0-4b0c-92a9-32901c3738f7"],
Cell[15096912, 427086, 417431, 12140, 3822, "Output", "ExpressionUUID" -> \
"fe2e7939-b71e-40e5-b1d8-71786b49f046"]
}, Open  ]],
Cell[15514358, 439229, 177, 3, 29, "Input", "ExpressionUUID" -> \
"9127f300-1f57-4a46-827e-01627ae08258"],
Cell[CellGroupData[{
Cell[15514560, 439236, 1669, 37, 102, "Input", "ExpressionUUID" -> \
"0917a4e1-d8e8-40ff-aeb5-46ffd067d14e"],
Cell[15516232, 439275, 493, 14, 23, "Print", "ExpressionUUID" -> \
"87e3f744-3feb-435a-be78-21ec255e9f59"],
Cell[15516728, 439291, 265537, 7720, 2844, "Output", "ExpressionUUID" -> \
"3c3347e9-66dc-44f7-af4b-93e04b2073a7"]
}, Open  ]],
Cell[15782280, 447014, 181, 3, 29, "Input", "ExpressionUUID" -> \
"7be0c200-8452-43c8-a16e-965473d3093f"],
Cell[CellGroupData[{
Cell[15782486, 447021, 1974, 45, 138, "Input", "ExpressionUUID" -> \
"51a89c42-5027-473c-95c3-e6b95c948d98"],
Cell[15784463, 447068, 517, 14, 23, "Print", "ExpressionUUID" -> \
"4f54caa4-3a5f-4ad8-8bb5-3bdc2814cd95"],
Cell[15784983, 447084, 322518, 9352, 3364, "Output", "ExpressionUUID" -> \
"62e29ab3-ace9-4f11-bbc3-a20ebbc3c4d4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16107538, 456441, 1822, 41, 120, "Input", "ExpressionUUID" -> \
"cb9e564b-f127-462b-8797-dae0c7374f22"],
Cell[16109363, 456484, 493, 14, 23, "Print", "ExpressionUUID" -> \
"a0d9fbb2-4a32-49d9-9e51-27e1bf9d63fd"],
Cell[16109859, 456500, 213426, 6199, 2592, "Output", "ExpressionUUID" -> \
"afe637af-f5f4-41b5-a0a8-f33fefb25788"]
}, Open  ]],
Cell[16323300, 462702, 159, 6, 97, "Section", "ExpressionUUID" -> \
"fd994986-96a0-4c19-8d75-c75c958ad1ce"],
Cell[CellGroupData[{
Cell[16323484, 462712, 1013, 19, 42, "Input", "ExpressionUUID" -> \
"2f9dc048-a72c-42af-ae67-8f0847c935d1"],
Cell[16324500, 462733, 684, 17, 23, "Print", "ExpressionUUID" -> \
"804c59df-e5d5-42e3-9b78-ad3bf953b8c5"],
Cell[16325187, 462752, 187250, 5255, 1564, "Output", "ExpressionUUID" -> \
"2b2bbecb-6bb6-4922-a20d-931dd8ed9c33"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16512474, 468012, 880, 18, 48, "Input", "ExpressionUUID" -> \
"57f847d4-dd43-4f0c-829a-3cc511bd8fe4"],
Cell[16513357, 468032, 614, 16, 23, "Print", "ExpressionUUID" -> \
"feda3966-0fa8-4e72-8906-593311a64158"],
Cell[16513974, 468050, 433075, 12265, 3102, "Output", "ExpressionUUID" -> \
"d2c38b9d-2f25-44e1-a048-b45e47dabdf0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16947086, 480320, 827, 18, 48, "Input", "ExpressionUUID" -> \
"d36d3bb9-a3e0-4079-9abc-2839a688bd31"],
Cell[16947916, 480340, 530, 14, 23, "Print", "ExpressionUUID" -> \
"42c45aa2-9314-48c8-9a3f-dfa3659be49c"],
Cell[16948449, 480356, 417546, 12035, 2917, "Output", "ExpressionUUID" -> \
"c42cc0b8-1cab-4857-854d-08855aef8335"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17366032, 492396, 715, 17, 48, "Input", "ExpressionUUID" -> \
"214bf111-4a67-4678-9bac-6151ec31a511"],
Cell[17366750, 492415, 488, 14, 23, "Print", "ExpressionUUID" -> \
"80e004f6-a9e1-4a71-8c6e-74ea573b8174"],
Cell[17367241, 492431, 271602, 7796, 2272, "Output", "ExpressionUUID" -> \
"a29b9e73-2caa-459e-ac80-458fba5e8761"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17638880, 500232, 1278, 26, 66, "Input", "ExpressionUUID" -> \
"9f1b3149-1e6c-45e4-acfa-e25e8db041a3"],
Cell[17640161, 500260, 749, 17, 23, "Print", "ExpressionUUID" -> \
"423c4078-4195-4493-836d-f8020f66dc65"]
}, Open  ]],
Cell[17640925, 500280, 546215, 15336, 4008, "Output", "ExpressionUUID" -> \
"9b068b71-0cb5-4912-a551-ad6a15870b4a"],
Cell[CellGroupData[{
Cell[18187165, 515620, 1036, 22, 48, "Input", "ExpressionUUID" -> \
"6ac01025-85f3-4a76-92f4-0c4ca3384553"],
Cell[18188204, 515644, 512, 14, 23, "Print", "ExpressionUUID" -> \
"55643729-05f8-4dfa-8f95-6439454053b1"],
Cell[18188719, 515660, 262521, 7572, 2424, "Output", "ExpressionUUID" -> \
"1bc8f8b6-5439-43ac-a7e4-c8bc7e5b192c"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[18451289, 523238, 1130, 24, 66, "Input", "ExpressionUUID" -> \
"53872b3e-553e-4f36-b484-90501040599f"],
Cell[18452422, 523264, 587, 15, 23, "Print", "ExpressionUUID" -> \
"86b2b423-84c7-41ee-be35-afb38220d3c8"],
Cell[18453012, 523281, 323245, 9349, 2928, "Output", "ExpressionUUID" -> \
"4fbda093-bc14-4093-9885-6eb9c8ca6dc0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18776294, 532635, 1206, 26, 66, "Input", "ExpressionUUID" -> \
"0500c5e8-3122-4ec2-abe8-fb1b142870ad"],
Cell[18777503, 532663, 534, 14, 23, "Print", "ExpressionUUID" -> \
"ac2e8d72-5347-4322-a419-1a550bb083e9"],
Cell[18778040, 532679, 266101, 7719, 2366, "Output", "ExpressionUUID" -> \
"9a863d6e-c48e-42e7-b4b6-1ca492eca04d"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19044178, 540403, 1247, 27, 66, "Input", "ExpressionUUID" -> \
"81a65493-59b0-44cc-8760-14fc9eeeff71"],
Cell[19045428, 540432, 541, 14, 23, "Print", "ExpressionUUID" -> \
"75f9aa7b-2c8d-4059-8b6e-501719aa4d4d"],
Cell[19045972, 540448, 120844, 3512, 1568, "Output", "ExpressionUUID" -> \
"b5540fa5-f67f-49f8-8f6d-2abcadc430b7"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19166853, 543965, 1017, 23, 66, "Input", "ExpressionUUID" -> \
"298d0292-6135-431d-86fa-61ababa3e30a"],
Cell[19167873, 543990, 563, 15, 23, "Print", "ExpressionUUID" -> \
"849f3a10-0ee2-499f-9c2c-62bd812124bb"],
Cell[19168439, 544007, 150185, 4347, 1480, "Output", "ExpressionUUID" -> \
"dfa1282a-551a-4fc3-b537-8df5deef259e"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19318661, 548359, 1300, 30, 84, "Input", "ExpressionUUID" -> \
"ac5742b9-25e9-4edc-9eb9-9e1482d8aef0"],
Cell[19319964, 548391, 493, 14, 23, "Print", "ExpressionUUID" -> \
"7d1bacf3-a33b-4283-9888-0f757528f6c2"],
Cell[19320460, 548407, 256229, 7457, 2240, "Output", "ExpressionUUID" -> \
"063e0009-4cd3-4135-a935-924fbf0098c2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19576726, 555869, 1104, 25, 66, "Input", "ExpressionUUID" -> \
"052345f1-4173-4da1-b352-6b20a0baafbb"],
Cell[19577833, 555896, 537, 14, 23, "Print", "ExpressionUUID" -> \
"5639a6b3-2c6a-45e3-b2e7-f62c7fe5475f"],
Cell[19578373, 555912, 148397, 4298, 1518, "Output", "ExpressionUUID" -> \
"89f26112-3d25-4ce3-beff-5ef89febe9c9"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19726807, 560215, 1383, 30, 84, "Input", "ExpressionUUID" -> \
"63395ef2-283c-4e4e-92b5-d34c307ab7df"],
Cell[19728193, 560247, 565, 15, 23, "Print", "ExpressionUUID" -> \
"bfdba4d4-8893-45f3-8f0e-c6e98b24367f"],
Cell[19728761, 560264, 233344, 6765, 2201, "Output", "ExpressionUUID" -> \
"3d2cabd6-3ef0-4dae-b621-ee4c20d5ae56"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19962142, 567034, 1318, 30, 84, "Input", "ExpressionUUID" -> \
"9fcc97c1-a9d5-45f7-bec6-d2f34fcf7827"],
Cell[19963463, 567066, 493, 14, 23, "Print", "ExpressionUUID" -> \
"685c1148-7683-4467-a264-3084a317ae43"],
Cell[19963959, 567082, 284017, 8239, 2532, "Output", "ExpressionUUID" -> \
"d7b3115e-1c96-4d02-a9e4-c72c2576d44b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20248013, 575326, 94, 0, 71, "Section", "ExpressionUUID" -> \
"782ac299-fd42-47f3-b652-194f7304b238"],
Cell[CellGroupData[{
Cell[20248132, 575330, 1131, 23, 48, "Input", "ExpressionUUID" -> \
"91367ffd-1b0b-4bf4-bbe8-26ec6b0f6e34"],
Cell[20249266, 575355, 747, 17, 23, "Print", "ExpressionUUID" -> \
"f6ad40c0-403c-4cb1-aea3-8f6237f0866b"],
Cell[20250016, 575374, 13253, 343, 292, "Output", "ExpressionUUID" -> \
"3e8adfa8-6b52-42c6-9e10-d15550a899ff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20263306, 575722, 1084, 24, 66, "Input", "ExpressionUUID" -> \
"00f2d204-2b2f-4544-bcc1-a524f3f1ea2a"],
Cell[20264393, 575748, 511, 14, 23, "Print", "ExpressionUUID" -> \
"effcf1f5-172a-4a1b-92d2-662eae55e220"],
Cell[20264907, 575764, 413269, 12064, 2940, "Output", "ExpressionUUID" -> \
"5f7120df-c68f-419d-9083-62c0add57103"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20678213, 587833, 407, 9, 29, "Input", "ExpressionUUID" -> \
"07735f2f-f8b5-44c8-8ba9-4110195f4cd3"],
Cell[20678623, 587844, 512, 14, 23, "Print", "ExpressionUUID" -> \
"3cb8d079-0fd3-4431-8271-3c36acbf6fd7"],
Cell[20679138, 587860, 89212, 2561, 874, "Output", "ExpressionUUID" -> \
"7cea4297-b966-4e83-a29d-fbce2ad5cc46"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20768387, 590426, 994, 21, 66, "Input", "ExpressionUUID" -> \
"6b938227-965d-4b0b-90ab-1eba69aa691e"],
Cell[20769384, 590449, 609, 15, 23, "Print", "ExpressionUUID" -> \
"7fa76943-711c-4c7f-9bbb-192ee3bb2a2f"],
Cell[20769996, 590466, 182854, 5310, 1536, "Output", "ExpressionUUID" -> \
"7b49d791-2d78-4ea0-9d22-1e51d43535f9"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[20952899, 595782, 451, 9, 29, "Input", "ExpressionUUID" -> \
"c1847ea3-fafb-4acd-af82-292514fda463"],
Cell[20953353, 595793, 563, 15, 23, "Print", "ExpressionUUID" -> \
"c30c97b3-db4d-4944-a095-d511c45bda9c"],
Cell[20953919, 595810, 78729, 2253, 724, "Output", "ExpressionUUID" -> \
"51b547be-6196-482a-95f2-c44bdfa83dd3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21032685, 598068, 523, 12, 29, "Input", "ExpressionUUID" -> \
"f7fbd80d-a383-4491-9e36-f84a3373db45"],
Cell[21033211, 598082, 513, 14, 23, "Print", "ExpressionUUID" -> \
"78f26a53-c201-42fd-888e-7249f1dd5a8e"]
}, Open  ]],
Cell[21033739, 598099, 1394054, 39592, 8052, "Output", "ExpressionUUID" -> \
"87a974b3-becd-49b8-9e72-5c46d8916650"],
Cell[CellGroupData[{
Cell[22427818, 637695, 881, 18, 48, "Input", "ExpressionUUID" -> \
"6725efba-b661-4e4a-be52-af8f8f3d3d97"],
Cell[22428702, 637715, 607, 15, 23, "Print", "ExpressionUUID" -> \
"afdb230c-d8b9-46a1-832d-69216239a33f"]
}, Open  ]],
Cell[22429324, 637733, 571952, 16170, 3568, "Output", "ExpressionUUID" -> \
"fa2cec5b-8e3b-4e23-8e7c-2d174c617152"],
Cell[23001279, 653905, 152, 3, 29, "Input", "ExpressionUUID" -> \
"eb6ea702-8360-4677-bf94-f9adb5798f1b"]
}
]
*)

